@@ -161,7 +161,7 @@ void usage(void)
 
 
 static
-void printInfo(int gridtype, int vdate, int vtime, char *varname, double level,
+void printInfo(int vdate, int vtime, char *varname, double level,
 	       int datasize, int number, int nmiss, double missval, const double *data, int vardis)
 {
   static int rec = 0;
@@ -190,13 +190,6 @@ void printInfo(int gridtype, int vdate, int vtime, char *varname, double level,
 
   fprintf(stdout, "%7d :", nmiss);
 
-  /*
-  if ( gridtype == GRID_SPECTRAL )
-    {
-      fprintf(stdout, "            %#12.5g\n", data[0]);
-    }
-  else
-  */
   if ( number == CDI_REAL )
     {
       if ( nmiss > 0 )
@@ -272,20 +265,22 @@ void printInfo(int gridtype, int vdate, int vtime, char *varname, double level,
 
 const char * tunit2str(int tunits)
 {
-  if      ( tunits == TUNIT_YEAR )    return ("years");
-  else if ( tunits == TUNIT_MONTH )   return ("months");
-  else if ( tunits == TUNIT_DAY )     return ("days");
-  else if ( tunits == TUNIT_12HOURS ) return ("12hours");
-  else if ( tunits == TUNIT_6HOURS )  return ("6hours");
-  else if ( tunits == TUNIT_3HOURS )  return ("3hours");
-  else if ( tunits == TUNIT_HOUR )    return ("hours");
-  else if ( tunits == TUNIT_MINUTE )  return ("minutes");
-  else if ( tunits == TUNIT_SECOND )  return ("seconds");
-  else                                return ("unknown");
+  if      ( tunits == TUNIT_YEAR )       return ("years");
+  else if ( tunits == TUNIT_MONTH )      return ("months");
+  else if ( tunits == TUNIT_DAY )        return ("days");
+  else if ( tunits == TUNIT_12HOURS )    return ("12hours");
+  else if ( tunits == TUNIT_6HOURS )     return ("6hours");
+  else if ( tunits == TUNIT_3HOURS )     return ("3hours");
+  else if ( tunits == TUNIT_HOUR )       return ("hours");
+  else if ( tunits == TUNIT_30MINUTES )  return ("30minutes");
+  else if ( tunits == TUNIT_QUARTER )    return ("15minutes");
+  else if ( tunits == TUNIT_MINUTE )     return ("minutes");
+  else if ( tunits == TUNIT_SECOND )     return ("seconds");
+  else                                   return ("unknown");
 }
 
 
-const char * calendar2str(int calendar)
+const char* calendar2str(int calendar)
 {
   if      ( calendar == CALENDAR_STANDARD )  return ("standard");
   else if ( calendar == CALENDAR_PROLEPTIC ) return ("proleptic_gregorian");
@@ -295,31 +290,42 @@ const char * calendar2str(int calendar)
   else                                       return ("unknown");
 }
 
+static
+void limit_string_length(char* string)
+{
+  size_t len = strlen(string);
+
+  if ( len > 10 )
+    {
+      for ( size_t i = 3; i < len; ++i )
+	if ( string[i] == ' ' )
+	  {
+	    string[i] = 0;
+	    break;
+	  }
+    }
+}
+
 static
 void printShortinfo(int streamID, int vlistID, int vardis)
 {
   int varID;
   int gridsize = 0;
   int gridID, zaxisID, param;
-  int zaxistype, ltype;
   int vdate, vtime;
-  int nrecs, nvars, nzaxis, ntsteps;
-  int levelID, levelsize;
-  int tsID, ntimeout;
+  int nvars, ntsteps;
+  int levelsize;
   int tsteptype, taxisID;
-  int nbyte, nbyte0;
-  int index;
+  char tmpname[CDI_MAX_NAME];
   char varname[CDI_MAX_NAME];
-  char longname[CDI_MAX_NAME];
-  char units[CDI_MAX_NAME];
-  double level;
-  char *modelptr, *instptr;
+  const char *modelptr, *instptr;
   int datatype;
   int year, month, day, hour, minute, second;
   char pstr[4];
   char paramstr[32];
 
-      printf("   File format: ");
+      fprintf(stdout, "   File format");
+      fprintf(stdout, " : ");
       printFiletype(streamID, vlistID);
 
       if ( vardis )
@@ -341,17 +347,17 @@ void printShortinfo(int streamID, int vlistID, int vardis)
 
 	  /* institute info */
 	  instptr = institutInqNamePtr(vlistInqVarInstitut(vlistID, varID));
-	  if ( instptr )
-	    fprintf(stdout, "%-8s ", instptr);
-	  else
-	    fprintf(stdout, "unknown  ");
+	  strcpy(tmpname, "unknown");
+	  if ( instptr ) strcpy(tmpname, instptr);
+	  limit_string_length(tmpname);
+	  fprintf(stdout, "%-8s ", tmpname);
 
 	  /* source info */
 	  modelptr = modelInqNamePtr(vlistInqVarModel(vlistID, varID));
-	  if ( modelptr )
-	    fprintf(stdout, "%-8s ", modelptr);
-	  else
-	    fprintf(stdout, "unknown  ");
+	  strcpy(tmpname, "unknown");
+	  if ( modelptr ) strcpy(tmpname, modelptr);
+	  limit_string_length(tmpname);
+	  fprintf(stdout, "%-8s ", tmpname);
 
 	  /* tsteptype */
 	  tsteptype = vlistInqVarTsteptype(vlistID, varID);
@@ -361,7 +367,10 @@ void printShortinfo(int streamID, int vlistID, int vardis)
 	  else if ( tsteptype == TSTEP_INSTANT3 ) fprintf(stdout, "%-8s ", "instant");
 	  else if ( tsteptype == TSTEP_MIN      ) fprintf(stdout, "%-8s ", "min");
 	  else if ( tsteptype == TSTEP_MAX      ) fprintf(stdout, "%-8s ", "max");
+	  else if ( tsteptype == TSTEP_AVG      ) fprintf(stdout, "%-8s ", "avg");
 	  else if ( tsteptype == TSTEP_ACCUM    ) fprintf(stdout, "%-8s ", "accum");
+	  else if ( tsteptype == TSTEP_RANGE    ) fprintf(stdout, "%-8s ", "range");
+	  else if ( tsteptype == TSTEP_DIFF     ) fprintf(stdout, "%-8s ", "diff");
 	  else                                    fprintf(stdout, "%-8s ", "unknown");
 
 	  /* layer info */
@@ -402,93 +411,26 @@ void printShortinfo(int streamID, int vlistID, int vardis)
 
 	  cdiParamToString(param, paramstr, sizeof(paramstr));
 
-	  if ( vardis ) vlistInqVarName(vlistID, varID, varname);
-
-          if ( vardis )
-	    fprintf(stdout, "%-11s", varname);
+	  if (vardis)
+            {
+              vlistInqVarName(vlistID, varID, varname);
+              fprintf(stdout, "%-11s", varname);
+            }
 	  else
 	    fprintf(stdout, "%-11s", paramstr);
 
 	  fprintf(stdout, "\n");
 	}
 
-      fprintf(stdout, "   Grid coordinates :\n");
+      fprintf(stdout, "   Grid coordinates");
+      fprintf(stdout, " :\n");
+
       printGridInfo(vlistID);
 
-      nzaxis = vlistNzaxis(vlistID);
-      fprintf(stdout, "   Vertical coordinates :\n");
-      for ( index = 0; index < nzaxis; index++)
-	{
-	  zaxisID   = vlistZaxis(vlistID, index);
-	  zaxistype = zaxisInqType(zaxisID);
-	  ltype     = zaxisInqLtype(zaxisID);
-	  levelsize = zaxisInqSize(zaxisID);
-	  /* zaxisInqLongname(zaxisID, longname); */
-	  zaxisName(zaxistype, longname);
-	  longname[18] = 0;
-	  zaxisInqUnits(zaxisID, units);
-	  units[12] = 0;
-	  if ( zaxistype == ZAXIS_GENERIC && ltype != 0 )
-	    nbyte0    = fprintf(stdout, "  %4d : %-11s  (ltype=%3d) : ", vlistZaxisIndex(vlistID, zaxisID)+1, longname, ltype);
-	  else
-	    nbyte0    = fprintf(stdout, "  %4d : %-18s %5s : ", vlistZaxisIndex(vlistID, zaxisID)+1, longname, units);
-	  nbyte = nbyte0;
-	  for ( levelID = 0; levelID < levelsize; levelID++ )
-	    {
-	      if ( nbyte > MAXCHARS )
-		{
-		  fprintf(stdout, "\n");
-		  fprintf(stdout, "%*s", nbyte0, "");
-		  nbyte = nbyte0;
-		}
-	      level = zaxisInqLevel(zaxisID, levelID);
-	      nbyte += fprintf(stdout, "%.9g ", level);
-	    }
-	  fprintf(stdout, "\n");
-	  if ( zaxisInqLbounds(zaxisID, NULL) && zaxisInqUbounds(zaxisID, NULL) )
-	    {
-	      double level1, level2;
-	      nbyte = nbyte0;
-	      fprintf(stdout, "%33s : ", "bounds");
-	      for ( levelID = 0; levelID < levelsize; levelID++ )
-		{
-		  if ( nbyte > MAXCHARS )
-		    {
-		      fprintf(stdout, "\n");
-		      fprintf(stdout, "%*s", nbyte0, "");
-		      nbyte = nbyte0;
-		    }
-		  level1 = zaxisInqLbound(zaxisID, levelID);
-		  level2 = zaxisInqUbound(zaxisID, levelID);
-		  nbyte += fprintf(stdout, "%.9g-%.9g ", level1, level2);
-		}
-	      fprintf(stdout, "\n");
-	    }
+      fprintf(stdout, "   Vertical coordinates");
+      fprintf(stdout, " :\n");
 
-          if ( zaxistype == ZAXIS_REFERENCE )
-            {
-              int number   = zaxisInqNumber(zaxisID);
-
-              if ( number > 0 )
-                {
-                  fprintf(stdout, "%33s : ", "zaxis");
-                  fprintf(stdout, "number = %d\n", number);
-                }
-
-              char uuidOfVGrid[17];
-              zaxisInqUUID(zaxisID, uuidOfVGrid);
-              if ( uuidOfVGrid[0] != 0 )
-                {
-                  char uuidOfVGridStr[37];
-                  uuid2str(uuidOfVGrid, uuidOfVGridStr);
-                  if ( uuidOfVGridStr[0] != 0  && strlen(uuidOfVGridStr) == 36 )
-                    {
-                      fprintf(stdout, "%33s : ", "uuid");
-                      fprintf(stdout, "%s\n", uuidOfVGridStr);
-                    }
-                }
-            }
-	}
+      printZaxisInfo(vlistID);
 
       taxisID = vlistInqTaxis(vlistID);
       ntsteps = vlistNtsteps(vlistID);
@@ -530,27 +472,8 @@ void printShortinfo(int streamID, int vlistID, int vardis)
 
 	  fprintf(stdout, "  YYYY-MM-DD hh:mm:ss  YYYY-MM-DD hh:mm:ss  YYYY-MM-DD hh:mm:ss  YYYY-MM-DD hh:mm:ss\n");
 
-	  ntimeout = 0;
-	  tsID = 0;
-	  while ( (nrecs = streamInqTimestep(streamID, tsID)) )
-	    {
-	      if ( ntimeout == 4 )
-		{
-		  ntimeout = 0;
-		  fprintf(stdout, "\n");
-		}
+	  printTimesteps(streamID, taxisID, 0);
 
-	      vdate = taxisInqVdate(taxisID);
-	      vtime = taxisInqVtime(taxisID);
-
-	      cdiDecodeDate(vdate, &year, &month, &day);
-	      cdiDecodeTime(vtime, &hour, &minute, &second);
-
-	      fprintf(stdout, " %5.4d-%2.2d-%2.2d %2.2d:%2.2d:%2.2d",
-		      year, month, day, hour, minute, second);
-	      ntimeout++;
-	      tsID++;
-	    }
 	  fprintf(stdout, "\n");
 	}
 }
@@ -786,7 +709,7 @@ int main(int argc, char *argv[])
   int Move = 0;
   int Record = 0;
   int Debug = 0;
-  int Quiet  = 0;
+  /* int Quiet  = 0; */
   int Vardis = 0;
   int Version = 0;
   int Longinfo = 0;
@@ -827,7 +750,7 @@ int main(int argc, char *argv[])
 	  Move = 1;
 	  break;
 	case 'q':
-	  Quiet = 1;
+	  /* Quiet = 1; */
 	  break;
 	case 'R':
 	  cdiDefGlobal("REGULARGRID", 1);
@@ -899,7 +822,6 @@ int main(int argc, char *argv[])
       int tsID;
       int ntsteps = 0;
       int taxisID1, taxisID2 = CDI_UNDEFID;
-      int gridtype;
       int vlistID1, vlistID2 = CDI_UNDEFID;
 
       streamID1 = streamOpenRead(fname1);
@@ -922,8 +844,9 @@ int main(int argc, char *argv[])
       taxisID1 = vlistInqTaxis(vlistID1);
       ntsteps = vlistNtsteps(vlistID1);
 
-      if ( Debug ) fprintf(stderr, "nvars   = %d\n", nvars);
-      if ( Debug ) fprintf(stderr, "ntsteps = %d\n", ntsteps);
+      if (Debug)
+        fprintf(stderr, "nvars   = %d\n"
+                "ntsteps = %d\n", nvars, ntsteps);
 
       if ( fname2 )
         {
@@ -971,16 +894,15 @@ int main(int argc, char *argv[])
 	}
 
       if ( vlistNumber(vlistID1) != CDI_REAL ) datasize *= 2;
-      data = (double *) malloc(datasize*sizeof(double));
+      data = (double *) malloc((size_t)datasize * sizeof (double));
 
       /*
 	nts = cdiInqTimeSize(streamID1);
       */
-      if ( Debug )
-	printf("nts = %d\n", nts);
-
-      if ( Debug )
-	printf("streamID1 = %d, streamID2 = %d\n", streamID1, streamID2);
+      if (Debug)
+	printf("nts = %d\n"
+               "streamID1 = %d, streamID2 = %d\n",
+               nts, streamID1, streamID2);
 
       if ( Shortinfo )
 	{
@@ -1023,13 +945,12 @@ int main(int argc, char *argv[])
 		  printf("varID=%d, param=%d, gridID=%d, zaxisID=%d levelID=%d\n",
 			 varID, param, gridID, zaxisID, levelID);
 		  */
-		  gridtype = gridInqType(gridID);
 		  gridsize = gridInqSize(gridID);
 		  level    = zaxisInqLevel(zaxisID, levelID);
 		  missval  = vlistInqVarMissval(vlistID1, varID);
 
 		  if ( Info )
-		    printInfo(gridtype, vdate, vtime, varname, level, gridsize, number, nmiss, missval, data, Vardis);
+		    printInfo(vdate, vtime, varname, level, gridsize, number, nmiss, missval, data, Vardis);
 
 		  if ( fname2 )
 		    {
@@ -1061,7 +982,6 @@ int main(int argc, char *argv[])
 		    fprintf(stdout, "varID = %d param = %d gridID = %d zaxisID = %d\n",
 			    varID, param, gridID, zaxisID);
 
-		  gridtype = gridInqType(gridID);
 		  gridsize = gridInqSize(gridID);
 		  missval  = vlistInqVarMissval(vlistID1, varID);
 
@@ -1072,7 +992,7 @@ int main(int argc, char *argv[])
 		      streamReadVarSlice(streamID1, varID, levelID, data, &nmiss);
 
 		      if ( Info )
-			printInfo(gridtype, vdate, vtime, varname, level, gridsize, number, nmiss, missval, data, Vardis);
+			printInfo(vdate, vtime, varname, level, gridsize, number, nmiss, missval, data, Vardis);
 
 		      if ( fname2 )
 			streamWriteVarSlice(streamID2, varID, levelID, data, nmiss);
