@@ -7,12 +7,13 @@
 #ifndef  CDI_H_
 #define  CDI_H_
 
+#include <stdio.h>
 #include <sys/types.h>
-
+/*
 #if defined(__cplusplus)
 extern "C" {
 #endif
-
+*/
 #define  CDI_MAX_NAME           256   /* max length of a name                 */
 
 #define  CDI_UNDEFID             -1
@@ -40,6 +41,7 @@ extern "C" {
 
 /* File types */
 
+#define  FILETYPE_UNDEF          -1   /* Unknown/not yet defined file type */
 #define  FILETYPE_GRB             1   /* File type GRIB                       */
 #define  FILETYPE_GRB2            2   /* File type GRIB version 2             */
 #define  FILETYPE_NC              3   /* File type netCDF                     */
@@ -191,19 +193,21 @@ extern "C" {
 
 #define  TAXIS_ABSOLUTE           1
 #define  TAXIS_RELATIVE           2
+#define  TAXIS_FORECAST           3
 
 /* TUNIT types */
 
 #define  TUNIT_SECOND             1
 #define  TUNIT_MINUTE             2
-#define  TUNIT_HOUR               3
-#define  TUNIT_DAY                4
-#define  TUNIT_MONTH              5
-#define  TUNIT_YEAR               6
-#define  TUNIT_QUARTER            7
-#define  TUNIT_3HOURS             8
-#define  TUNIT_6HOURS             9
-#define  TUNIT_12HOURS           10
+#define  TUNIT_QUARTER            3
+#define  TUNIT_30MINUTES          4
+#define  TUNIT_HOUR               5
+#define  TUNIT_3HOURS             6
+#define  TUNIT_6HOURS             7
+#define  TUNIT_12HOURS            8
+#define  TUNIT_DAY                9
+#define  TUNIT_MONTH             10
+#define  TUNIT_YEAR              11
 
 /* CALENDAR types */
 
@@ -214,6 +218,8 @@ extern "C" {
 #define  CALENDAR_366DAYS         4
 #define  CALENDAR_NONE            5
 
+/* number of unsigned char needed to store UUID */
+#define  CDI_UUID_SIZE           16
 
 /* CDI control routines */
 
@@ -223,7 +229,7 @@ char   *cdiStringError(int cdiErrno);
 
 void    cdiDebug(int debug);
 
-char   *cdiLibraryVersion(void);
+const char *cdiLibraryVersion(void);
 void    cdiPrintVersion(void);
 
 int     cdiHaveFiletype(int filetype);
@@ -277,6 +283,7 @@ void    streamDefVlist(int streamID, int vlistID);
 
 /*      streamInqVlist: Get the Vlist of a stream */
 int     streamInqVlist(int streamID);
+/*      PIO: */
 int     streamInqVlistIDorig(int streamID);
 
 /*      streamInqFiletype: Get the filetype */
@@ -291,24 +298,24 @@ int     streamInqByteorder(int streamID);
 /*      streamDefCompType: Define compression type */
 void    streamDefCompType(int streamID, int comptype);
 
-/*      streamDefCompLevel: Define compression level */
-void    streamDefCompLevel(int streamID, int complevel);
-
 /*      streamInqCompType: Get compression type */
 int     streamInqCompType(int streamID);
 
+/*      streamDefCompLevel: Define compression level */
+void    streamDefCompLevel(int streamID, int complevel);
+
 /*      streamInqCompLevel: Get compression level */
 int     streamInqCompLevel(int streamID);
 
 /*      streamDefTimestep: Define time step */
 int     streamDefTimestep(int streamID, int tsID);
 
-/* query currently set timestep id  */
-int streamInqCurTimestepID(int streamID);
-
 /*      streamInqTimestep: Get time step */
 int     streamInqTimestep(int streamID, int tsID);
 
+/*      PIO: query currently set timestep id  */
+int     streamInqCurTimestepID(int streamID);
+
 char   *streamFilename(int streamID);
 char   *streamFilesuffix(int filetype);
 int     streamNtsteps(int streamID);
@@ -318,34 +325,36 @@ int     streamInqNvars ( int streamID );
 
 /* STREAM var I/O routines */
 
-/*      streamReadVar: Read a variable */
-void    streamReadVar(int streamID, int varID, double *data_vec, int *nmiss);
-
 /*      streamWriteVar: Write a variable */
 void    streamWriteVar(int streamID, int varID, const double *data_vec, int nmiss);
 void    streamWriteVarF(int streamID, int varID, const float *data_vec, int nmiss);
 
-/*      streamReadVarSlice: Read a horizontal slice of a variable */
-void    streamReadVarSlice(int streamID, int varID, int levelID, double *data_vec, int *nmiss);
+/*      streamReadVar: Read a variable */
+void    streamReadVar(int streamID, int varID, double *data_vec, int *nmiss);
+void    streamReadVarF(int streamID, int varID, float *data_vec, int *nmiss);
 
 /*      streamWriteVarSlice: Write a horizontal slice of a variable */
 void    streamWriteVarSlice(int streamID, int varID, int levelID, const double *data_vec, int nmiss);
 void    streamWriteVarSliceF(int streamID, int varID, int levelID, const float *data_vec, int nmiss);
 
+/*      streamReadVarSlice: Read a horizontal slice of a variable */
+void    streamReadVarSlice(int streamID, int varID, int levelID, double *data_vec, int *nmiss);
+void    streamReadVarSliceF(int streamID, int varID, int levelID, float *data_vec, int *nmiss);
+
 void    streamWriteVarChunk(int streamID, int varID, const int rect[][2],
                             const double *data_vec, int nmiss);
 
 
 /* STREAM record I/O routines */
 
-void    streamInqRecord(int streamID, int *varID, int *levelID);
 void    streamDefRecord(int streamID, int  varID, int  levelID);
-void    streamReadRecord(int streamID, double *data_vec, int *nmiss);
+void    streamInqRecord(int streamID, int *varID, int *levelID);
 void    streamWriteRecord(int streamID, const double *data_vec, int nmiss);
 void    streamWriteRecordF(int streamID, const float *data_vec, int nmiss);
+void    streamReadRecord(int streamID, double *data_vec, int *nmiss);
 void    streamCopyRecord(int streamIDdest, int streamIDsrc);
 
-void    streamInqGinfo(int streamID, int *intnum, float *fltnum, off_t *bignum);
+void    streamInqGRIBinfo(int streamID, int *intnum, float *fltnum, off_t *bignum);
 
 /* VLIST routines */
 
@@ -428,8 +437,8 @@ int     vlistInqVarZaxis(int vlistID, int varID);
 /* used in MPIOM */
 int     vlistInqVarID(int vlistID, int code);
 
-int     vlistInqVarTsteptype(int vlistID, int varID);
 void    vlistDefVarTsteptype(int vlistID, int varID, int tsteptype);
+int     vlistInqVarTsteptype(int vlistID, int varID);
 
 void    vlistDefVarCompType(int vlistID, int varID, int comptype);
 int     vlistInqVarCompType(int vlistID, int varID);
@@ -518,6 +527,9 @@ int     vlistInqVarTimaccu(int vlistID, int varID);
 void    vlistDefVarTypeOfGeneratingProcess(int vlistID, int varID, int typeOfGeneratingProcess);
 int     vlistInqVarTypeOfGeneratingProcess(int vlistID, int varID);
 
+void    vlistDefVarProductDefinitionTemplate(int vlistID, int varID, int productDefinitionTemplate);
+int     vlistInqVarProductDefinitionTemplate(int vlistID, int varID);
+
 int     vlistInqVarSize(int vlistID, int varID);
 
 void    vlistDefIndex(int vlistID, int varID, int levID, int index);
@@ -577,7 +589,7 @@ int     vlistInqAttTxt(int vlistID, int varID, const char *name, int mlen, char
 /* GRID routines */
 
 void    gridName(int gridtype, char *gridname);
-char   *gridNamePtr(int gridtype);
+const char *gridNamePtr(int gridtype);
 
 void    gridCompress(int gridID);
 
@@ -637,45 +649,45 @@ int     gridInqYvals(int gridID, double *yvals_vec);
 /*      gridDefXname: Define the name of a X-axis */
 void    gridDefXname(int gridID, const char *xname);
 
-/*      gridDefXlongname: Define the longname of a X-axis  */
-void    gridDefXlongname(int gridID, const char *xlongname);
-
-/*      gridDefXunits: Define the units of a X-axis */
-void    gridDefXunits(int gridID, const char *xunits);
-
-/*      gridDefYname: Define the name of a Y-axis */
-void    gridDefYname(int gridID, const char *yname);
-
-/*      gridDefYlongname: Define the longname of a Y-axis */
-void    gridDefYlongname(int gridID, const char *ylongname);
-
-/*      gridDefYunits: Define the units of a Y-axis */
-void    gridDefYunits(int gridID, const char *yunits);
-
 /*      gridInqXname: Get the name of a X-axis */
 void    gridInqXname(int gridID, char *xname);
 
+/*      gridDefXlongname: Define the longname of a X-axis  */
+void    gridDefXlongname(int gridID, const char *xlongname);
+
 /*      gridInqXlongname: Get the longname of a X-axis */
 void    gridInqXlongname(int gridID, char *xlongname);
 
-/*      gridInqXstdname: Get the standard name of a X-axis */
-void    gridInqXstdname(int gridID, char *xstdname);
+/*      gridDefXunits: Define the units of a X-axis */
+void    gridDefXunits(int gridID, const char *xunits);
 
 /*      gridInqXunits: Get the units of a X-axis */
 void    gridInqXunits(int gridID, char *xunits);
 
+/*      gridDefYname: Define the name of a Y-axis */
+void    gridDefYname(int gridID, const char *yname);
+
 /*      gridInqYname: Get the name of a Y-axis */
 void    gridInqYname(int gridID, char *yname);
 
+/*      gridDefYlongname: Define the longname of a Y-axis */
+void    gridDefYlongname(int gridID, const char *ylongname);
+
 /*      gridInqYlongname: Get the longname of a Y-axis */
 void    gridInqYlongname(int gridID, char *ylongname);
 
-/*      gridInqYstdname: Get the standard name of a Y-axis */
-void    gridInqYstdname(int gridID, char *ystdname);
+/*      gridDefYunits: Define the units of a Y-axis */
+void    gridDefYunits(int gridID, const char *yunits);
 
 /*      gridInqYunits: Get the units of a Y-axis */
 void    gridInqYunits(int gridID, char *yunits);
 
+/*      gridInqXstdname: Get the standard name of a X-axis */
+void    gridInqXstdname(int gridID, char *xstdname);
+
+/*      gridInqYstdname: Get the standard name of a Y-axis */
+void    gridInqYstdname(int gridID, char *ystdname);
+
 /*      gridDefPrec: Define the precision of a Grid */
 void    gridDefPrec(int gridID, int prec);
 
@@ -693,23 +705,23 @@ double  gridInqYinc(int gridID);
 
 int     gridIsCircular(int gridID);
 int     gridIsRotated(int gridID);
-double  gridInqXpole(int gridID);
 void    gridDefXpole(int gridID, double xpole);
-double  gridInqYpole(int gridID);
+double  gridInqXpole(int gridID);
 void    gridDefYpole(int gridID, double ypole);
-double  gridInqAngle(int gridID);
+double  gridInqYpole(int gridID);
 void    gridDefAngle(int gridID, double angle);
-void    gridDefTrunc(int gridID, int trunc);
+double  gridInqAngle(int gridID);
 int     gridInqTrunc(int gridID);
+void    gridDefTrunc(int gridID, int trunc);
 /* Hexagonal GME grid */
-int     gridInqGMEnd(int gridID);
 void    gridDefGMEnd(int gridID, int nd);
-int     gridInqGMEni(int gridID);
+int     gridInqGMEnd(int gridID);
 void    gridDefGMEni(int gridID, int ni);
-int     gridInqGMEni2(int gridID);
+int     gridInqGMEni(int gridID);
 void    gridDefGMEni2(int gridID, int ni2);
-int     gridInqGMEni3(int gridID);
+int     gridInqGMEni2(int gridID);
 void    gridDefGMEni3(int gridID, int ni3);
+int     gridInqGMEni3(int gridID);
 
 /* Reference of an unstructured grid */
 
@@ -732,10 +744,10 @@ void    gridDefReference(int gridID, const char *reference);
 int     gridInqReference(int gridID, char *reference);
 
 /*      gridDefUUID: Define the UUID of an unstructured grid */
-void    gridDefUUID(int gridID, const char *uuid_cbuf);
+void    gridDefUUID(int gridID, const unsigned char uuid[CDI_UUID_SIZE]);
 
 /*      gridInqUUID: Get the UUID of an unstructured grid */
-void    gridInqUUID(int gridID, char *uuid_cbuf);
+void    gridInqUUID(int gridID, unsigned char uuid[CDI_UUID_SIZE]);
 
 
 /* Lambert Conformal Conic grid (GRIB version) */
@@ -828,32 +840,32 @@ void    zaxisDefNumber(int gridID, int number);
 int     zaxisInqNumber(int gridID);
 
 /*      zaxisDefUUID: Define the UUID of a generalized Z-axis */
-void    zaxisDefUUID(int zaxisID, const char *uuid_cbuf);
+void    zaxisDefUUID(int zaxisID, const unsigned char uuid[CDI_UUID_SIZE]);
 
 /*      zaxisInqUUID: Get the UUID of a generalized Z-axis */
-void    zaxisInqUUID(int zaxisID, char *uuid_cbuf);
+void    zaxisInqUUID(int zaxisID, unsigned char uuid[CDI_UUID_SIZE]);
 
 /*      zaxisDefName: Define the name of a Z-axis */
 void    zaxisDefName(int zaxisID, const char *name);
 
-/*      zaxisDefLongname: Define the longname of a Z-axis */
-void    zaxisDefLongname(int zaxisID, const char *longname);
-
-/*      zaxisDefUnits: Define the units of a Z-axis */
-void    zaxisDefUnits(int zaxisID, const char *units);
-
 /*      zaxisInqName: Get the name of a Z-axis */
 void    zaxisInqName(int zaxisID, char *name);
 
+/*      zaxisDefLongname: Define the longname of a Z-axis */
+void    zaxisDefLongname(int zaxisID, const char *longname);
+
 /*      zaxisInqLongname: Get the longname of a Z-axis */
 void    zaxisInqLongname(int zaxisID, char *longname);
 
-/*      zaxisInqStdname: Get the standard name of a Z-axis */
-void    zaxisInqStdname(int zaxisID, char *stdname);
+/*      zaxisDefUnits: Define the units of a Z-axis */
+void    zaxisDefUnits(int zaxisID, const char *units);
 
 /*      zaxisInqUnits: Get the units of a Z-axis */
 void    zaxisInqUnits(int zaxisID, char *units);
 
+/*      zaxisInqStdname: Get the standard name of a Z-axis */
+void    zaxisInqStdname(int zaxisID, char *stdname);
+
 void    zaxisDefPrec(int zaxisID, int prec);
 int     zaxisInqPrec(int zaxisID);
 
@@ -868,14 +880,14 @@ void    zaxisDefVct(int zaxisID, int size, const double *vct_vec);
 void    zaxisInqVct(int zaxisID, double *vct_vec);
 int     zaxisInqVctSize(int zaxisID);
 const double *zaxisInqVctPtr(int zaxisID);
-int     zaxisInqLbounds(int zaxisID, double *lbounds_vec);
-int     zaxisInqUbounds(int zaxisID, double *ubounds_vec);
-int     zaxisInqWeights(int zaxisID, double *weights_vec);
-double  zaxisInqLbound(int zaxisID, int index);
-double  zaxisInqUbound(int zaxisID, int index);
 void    zaxisDefLbounds(int zaxisID, const double *lbounds_vec);
+int     zaxisInqLbounds(int zaxisID, double *lbounds_vec);
+double  zaxisInqLbound(int zaxisID, int index);
 void    zaxisDefUbounds(int zaxisID, const double *ubounds_vec);
+int     zaxisInqUbounds(int zaxisID, double *ubounds_vec);
+double  zaxisInqUbound(int zaxisID, int index);
 void    zaxisDefWeights(int zaxisID, const double *weights_vec);
+int     zaxisInqWeights(int zaxisID, double *weights_vec);
 void    zaxisChangeType(int zaxisID, int zaxistype);
 
 /* TAXIS routines */
@@ -898,12 +910,36 @@ void    taxisDefVdate(int taxisID, int date);
 /*      taxisDefVtime: Define the verification time */
 void    taxisDefVtime(int taxisID, int time);
 
+/*      taxisInqVdate: Get the verification date */
+int     taxisInqVdate(int taxisID);
+
+/*      taxisInqVtime: Get the verification time */
+int     taxisInqVtime(int taxisID);
+
 /*      taxisDefRdate: Define the reference date */
 void    taxisDefRdate(int taxisID, int date);
 
-/*      taxisDefRtime: Define the reference date */
+/*      taxisDefRtime: Define the reference time */
 void    taxisDefRtime(int taxisID, int time);
 
+/*      taxisInqRdate: Get the reference date */
+int     taxisInqRdate(int taxisID);
+
+/*      taxisInqRtime: Get the reference time */
+int     taxisInqRtime(int taxisID);
+
+/*      taxisDefFdate: Define the forecast reference date */
+void    taxisDefFdate(int taxisID, int date);
+
+/*      taxisDefFtime: Define the forecast reference time */
+void    taxisDefFtime(int taxisID, int time);
+
+/*      taxisInqFdate: Get the forecast reference date */
+int     taxisInqFdate(int taxisID);
+
+/*      taxisInqFtime: Get the forecast reference time */
+int     taxisInqFtime(int taxisID);
+
 int     taxisHasBounds(int taxisID);
 
 void    taxisDeleteBounds(int taxisID);
@@ -919,29 +955,22 @@ void    taxisInqVtimeBounds(int taxisID, int *vtime_lb, int *vtime_ub);
 /*      taxisDefCalendar: Define the calendar */
 void    taxisDefCalendar(int taxisID, int calendar);
 
+/*      taxisInqCalendar: Get the calendar */
+int     taxisInqCalendar(int taxisID);
+
 void    taxisDefTunit(int taxisID, int tunit);
+int     taxisInqTunit(int taxisID);
+
+void    taxisDefForecastTunit(int taxisID, int tunit);
+int     taxisInqForecastTunit(int taxisID);
+
+void    taxisDefForecastPeriod(int taxisID, double fc_period);
+double  taxisInqForecastPeriod(int taxisID);
 
 void    taxisDefNumavg(int taxisID, int numavg);
 
 int     taxisInqType(int taxisID);
 
-/*      taxisInqVdate: Get the verification date */
-int     taxisInqVdate(int taxisID);
-
-/*      taxisInqVtime: Get the verification time */
-int     taxisInqVtime(int taxisID);
-
-/*      taxisInqRdate: Get the reference date */
-int     taxisInqRdate(int taxisID);
-
-/*      taxisInqRtime: Get the reference time */
-int     taxisInqRtime(int taxisID);
-
-/*      taxisInqCalendar: Get the calendar */
-int     taxisInqCalendar(int taxisID);
-
-int     taxisInqTunit(int taxisID);
-
 int     taxisInqNumavg(int taxisID);
 
 char   *tunitNamePtr(int tunitID);
@@ -954,7 +983,7 @@ int     institutInq(int center, int subcenter, const char *name, const char *lon
 int     institutInqNumber(void);
 int     institutInqCenter(int instID);
 int     institutInqSubcenter(int instID);
-char   *institutInqNamePtr(int instID);
+const char *institutInqNamePtr(int instID);
 char   *institutInqLongnamePtr(int instID);
 
 /* Model routines */
@@ -963,11 +992,12 @@ int     modelDef(int instID, int modelgribID, const char *name);
 int     modelInq(int instID, int modelgribID, char *name);
 int     modelInqInstitut(int modelID);
 int     modelInqGribID(int modelID);
-char   *modelInqNamePtr(int modelID);
+const char *modelInqNamePtr(int modelID);
 
 /* Table routines */
 
 void    tableWriteC(const char *filename, int tableID);
+void    tableFWriteC(FILE *ptfp, int tableID);
 void    tableWrite(const char *filename, int tableID);
 int     tableRead(const char *tablefile);
 int     tableDef(int modelID, int tablenum, const char *tablename);
@@ -988,9 +1018,9 @@ int     tableInqParName(int tableID, int code, char *name);
 int     tableInqParLongname(int tableID, int code, char *longname);
 int     tableInqParUnits(int tableID, int code, char *units);
 
-char   *tableInqParNamePtr(int tableID, int parID);
-char   *tableInqParLongnamePtr(int tableID, int parID);
-char   *tableInqParUnitsPtr(int tableID, int parID);
+const char *tableInqParNamePtr(int tableID, int parID);
+const char *tableInqParLongnamePtr(int tableID, int parID);
+const char *tableInqParUnitsPtr(int tableID, int parID);
 
 /* History routines */
 
@@ -998,10 +1028,14 @@ void    streamDefHistory(int streamID, int size, const char *history);
 int     streamInqHistorySize(int streamID);
 void    streamInqHistoryString(int streamID, char *history);
 
+
+void gribapiLibraryVersion(int *major_version, int *minor_version, int *revision_version);
+
+/*
 #if defined (__cplusplus)
 }
 #endif
-
+*/
 #endif  /* CDI_H_ */
 /*
  * Local Variables:
