@@ -1,6 +1,8 @@
 #ifndef _GRID_H
 #define _GRID_H
 
+#include "cdi.h"
+
 typedef unsigned char mask_t;
 
 typedef struct {
@@ -46,7 +48,7 @@ typedef struct {
   int     nd, ni, ni2, ni3;       /* parameter for GRID_GME         */
   int     number, position;       /* parameter for GRID_REFERENCE   */
   char   *reference;
-  char    uuid[17];               /* uuid for grid reference        */
+  unsigned char uuid[CDI_UUID_SIZE]; /* uuid for grid reference        */
   int     trunc;                  /* parameter for GRID_SPECTEAL    */
   int     nvertex;
   int    *rowlon;
@@ -57,6 +59,7 @@ typedef struct {
   int     np;                     /* number of parallels between a pole and the equator */
   int     locked;
   int     lcomplex;
+  int     hasdims;
   char    xname[CDI_MAX_NAME];
   char    yname[CDI_MAX_NAME];
   char    xlongname[CDI_MAX_NAME];
@@ -78,8 +81,8 @@ int gridSize(void);
 const double *gridInqXvalsPtr(int gridID);
 const double *gridInqYvalsPtr(int gridID);
 
-double *gridInqXboundsPtr(int gridID);
-double *gridInqYboundsPtr(int gridID);
+const double *gridInqXboundsPtr(int gridID);
+const double *gridInqYboundsPtr(int gridID);
 const double *gridInqAreaPtr(int gridID);
 
 int gridCompare(int gridID, grid_t grid);
@@ -87,6 +90,11 @@ int gridGenerate(grid_t grid);
 
 void gridGetIndexList( int, int * );
 
+void
+gridUnpack(char * unpackBuffer, int unpackBufferSize,
+           int * unpackBufferPos, int originNamespace, void *context,
+           int force_id);
+
 #endif
 /*
  * Local Variables:
