@@ -2,7 +2,6 @@
 #  include "config.h"
 #endif
 
-#ifdef USE_MPI
 #include <ctype.h>
 
 #include <yaxt.h>
@@ -22,6 +21,7 @@
 static void
 nullPackFunc(void *obj, void *buf, int size, int *pos, void *context)
 {
+  (void)obj; (void)buf; (void)size; (void)pos; (void)context;
 }
 
 
@@ -32,12 +32,14 @@ cdiPioClientStreamOpen(const char *filename, const char *filemode,
 {
   struct winHeaderEntry header;
   size_t filename_len;
+  (void)streamptr; (void)recordBufIsToBeCreated;
   if ( tolower ( * filemode ) == 'w' )
     {
       statusCode nspStatus = namespaceInqResStatus ();
       switch ( nspStatus )
         {
         case STAGE_DEFINITION:
+          streamptr->filetype = filetype;
           break;
         case STAGE_TIMELOOP:
           filename_len = strlen(filename);
@@ -54,6 +56,7 @@ cdiPioClientStreamOpen(const char *filename, const char *filemode,
                  " filename=%s, filetype=%d",
                  funcMap[(-1 - STREAMOPEN)], filename_len + 1, filename,
                  filetype);
+          streamptr->filetype = filetype;
           break;
         case STAGE_CLEANUP:
           xabort ( "TRANSITION TO IO PROCESSES ALREADY FINISHED." );
@@ -97,6 +100,7 @@ static void
 cdiPioClientStreamWriteVar_(int streamID, int varID, int memtype,
                             const void *data, int nmiss)
 {
+  (void)streamID; (void)varID; (void)memtype; (void)data; (void)nmiss;
   xabort("parallel writing requires explicit partition information,"
          " use streamWriteVarPart!");
 }
@@ -120,7 +124,7 @@ cdiPioClientStreamWriteVarChunk_(int streamID, int varID, int memtype,
   int varSize = varShape[0] * varShape[1] * varShape[2];
   xassert(varSize == size);
   Xt_idxlist chunkDesc
-    = xt_idxsection_new(0, ndims, varShapeXt, chunkShape, origin);
+    = xt_idxsection_new(0, (int)ndims, varShapeXt, chunkShape, origin);
   pioBufferPartData(streamID, varID, data, nmiss, chunkDesc);
   xt_idxlist_delete(chunkDesc);
 }
@@ -173,12 +177,14 @@ cdiPioClientStreamWriteScatteredVarPart(int streamID, int varID,
 static void
 cdiPioCdfDefTimestepNOP(stream_t *streamptr, int tsID)
 {
+  (void)streamptr; (void)tsID;
 }
 #endif
 
 static void
 cdiPioClientStreamNOP(stream_t *streamptr)
 {
+  (void)streamptr;
 }
 
 
@@ -187,6 +193,7 @@ cdiPioClientStreamClose(stream_t *streamptr, int recordBufIsToBeDeleted)
 {
   struct winHeaderEntry header;
   statusCode nspStatus = namespaceInqResStatus ();
+  (void)recordBufIsToBeDeleted;
   switch ( nspStatus )
     {
     case STAGE_DEFINITION:
@@ -249,7 +256,7 @@ cdiPioClientSetup(int *pioNamespace_, int *pioNamespace)
   *pioNamespace_ = *pioNamespace = namespaceNew();
   int callerCDINamespace = namespaceGetActive();
   namespaceSetActive(*pioNamespace_);
-  serializeSetMPI();
+  cdiPioSerializeSetMPI();
   namespaceSwitchSet(NSSWITCH_STREAM_OPEN_BACKEND,
                      NSSW_FUNC(cdiPioClientStreamOpen));
   namespaceSwitchSet(NSSWITCH_STREAM_DEF_VLIST_,
@@ -278,7 +285,6 @@ cdiPioClientSetup(int *pioNamespace_, int *pioNamespace)
 }
 
 
-#endif
 /*
  * Local Variables:
  * c-file-style: "Java"
