@@ -4,8 +4,6 @@
 #  include "config.h"
 #endif
 
-#ifdef USE_MPI
-
 #include "pio_server.h"
 
 
@@ -41,7 +39,6 @@
 #include "vlist_var.h"
 
 
-extern resOps streamOps;
 extern void arrayDestroy ( void );
 
 static struct
@@ -91,7 +88,7 @@ collDefBufferSizes()
   xassert(rxWin != NULL);
 
   nstreams = reshCountType ( &streamOps );
-  streamIndexList = xmalloc ( nstreams * sizeof ( streamIndexList[0] ));
+  streamIndexList = xmalloc((size_t)nstreams * sizeof (streamIndexList[0]));
   reshGetResHListOfType ( nstreams, streamIndexList, &streamOps );
   for ( streamNo = 0; streamNo < nstreams; streamNo++ )
     {
@@ -112,17 +109,18 @@ collDefBufferSizes()
                     int nProcsModel = commInqNProcsModel();
                     decoChunk =
                       (int)ceilf(cdiPIOpartInflate_
-                                 * (varSize + nProcsModel - 1)/nProcsModel);
+                                 * (float)(varSize + nProcsModel - 1)
+                                 / (float)nProcsModel);
                   }
                   xassert ( decoChunk > 0 );
-                  rxWin[modelID].size += decoChunk * sizeof (double)
+                  rxWin[modelID].size += (size_t)decoChunk * sizeof (double)
                     /* re-align chunks to multiple of double size */
                     + sizeof (double) - 1
                     /* one header for data record, one for
                      * corresponding part descriptor*/
                     + 2 * sizeof (struct winHeaderEntry)
                     /* FIXME: heuristic for size of packed Xt_idxlist */
-                    + sizeof (Xt_int) * decoChunk * 3;
+                    + sizeof (Xt_int) * (size_t)decoChunk * 3;
                   rxWin[modelID].dictSize += 2;
                 }
             }
@@ -171,9 +169,9 @@ serverWinCreate(void)
   xmpi ( MPI_Comm_group ( commCalc, &groupCalc ));
   xmpi ( MPI_Group_excl ( groupCalc, 1, ranks, &groupModel ));
 
-  rxWin = xcalloc(nProcsModel, sizeof (rxWin[0]));
+  rxWin = xcalloc((size_t)nProcsModel, sizeof (rxWin[0]));
   size_t totalBufferSize = collDefBufferSizes();
-  rxWin[0].buffer = xmalloc(totalBufferSize);
+  rxWin[0].buffer = (unsigned char*) xmalloc(totalBufferSize);
   size_t ofs = 0;
   for ( modelID = 1; modelID < nProcsModel; modelID++ )
     {
@@ -210,7 +208,7 @@ readFuncCall(struct winHeaderEntry *header)
       break;
     case STREAMOPEN:
       {
-        size_t filenamesz = funcArgs->newFile.fnamelen;
+        size_t filenamesz = (size_t)funcArgs->newFile.fnamelen;
         xassert ( filenamesz > 0 && filenamesz < MAXDATAFILENAME );
         const char *filename
           = (const char *)(rxWin[root].buffer + header->offset);
@@ -239,14 +237,14 @@ readFuncCall(struct winHeaderEntry *header)
       {
         MPI_Comm commCalc = commInqCommCalc ();
         int streamID = funcArgs->streamNewTimestep.streamID;
-        int nspTarget = namespaceResHDecode(streamID).nsp;
+        int originNamespace = namespaceResHDecode(streamID).nsp;
         streamID = namespaceAdaptKey2(streamID);
         int oldTaxisID
           = vlistInqTaxis(streamInqVlist(streamID));
         int position = header->offset;
         int changedTaxisID
           = taxisUnpack((char *)rxWin[root].buffer, (int)rxWin[root].size,
-                        &position, nspTarget, &commCalc, 0);
+                        &position, originNamespace, &commCalc, 0);
         taxis_t *oldTaxisPtr = taxisPtr(oldTaxisID);
         taxis_t *changedTaxisPtr = taxisPtr(changedTaxisID);
         ptaxisCopy(oldTaxisPtr, changedTaxisPtr);
@@ -266,7 +264,7 @@ resizeVarGatherBuf(int vlistID, int varID, double **buf, int *bufSize)
 {
   int size = vlistInqVarSize(vlistID, varID);
   if (size <= *bufSize) ; else
-    *buf = xrealloc(*buf, (*bufSize = size) * sizeof (buf[0][0]));
+    *buf = xrealloc(*buf, (size_t)(*bufSize = size) * sizeof (buf[0][0]));
 }
 
 static void
@@ -286,8 +284,8 @@ gatherArray(int root, int nProcsModel, int headerIdx,
     varShapeXt[i] = varShape[i];
   int varSize = varShape[0] * varShape[1] * varShape[2];
   struct Xt_offset_ext *partExts
-    = xmalloc(nProcsModel * sizeof (partExts[0]));
-  Xt_idxlist *part = xmalloc(nProcsModel * sizeof (part[0]));
+    = xmalloc((size_t)nProcsModel * sizeof (partExts[0]));
+  Xt_idxlist *part = xmalloc((size_t)nProcsModel * sizeof (part[0]));
   MPI_Comm commCalc = commInqCommCalc();
   {
     int nmiss_ = 0;
@@ -306,21 +304,21 @@ gatherArray(int root, int nProcsModel, int headerIdx,
                     rxWin[modelID].buffer)[headerIdx + 1].id == PARTDESCMARKER
                 && position > 0
                 && ((size_t)position
-                    >= sizeof (struct winHeaderEntry) * rxWin[modelID].dictSize)
+                    >= sizeof (struct winHeaderEntry) * (size_t)rxWin[modelID].dictSize)
                 && ((size_t)position < rxWin[modelID].size));
         part[modelID] = xt_idxlist_unpack(rxWin[modelID].buffer,
                                           (int)rxWin[modelID].size,
                                           &position, commCalc);
-        int partSize = xt_idxlist_get_num_indices(part[modelID]);
-        size_t charOfs = (rxWin[modelID].buffer
-                          + ((struct winHeaderEntry *)
-                             rxWin[modelID].buffer)[headerIdx].offset)
-          - rxWin[0].buffer;
+        unsigned partSize = (unsigned)xt_idxlist_get_num_indices(part[modelID]);
+        size_t charOfs = (size_t)((rxWin[modelID].buffer
+                                   + ((struct winHeaderEntry *)
+                                      rxWin[modelID].buffer)[headerIdx].offset)
+                                  - rxWin[0].buffer);
         xassert(charOfs % sizeof (double) == 0
                 && charOfs / sizeof (double) + partSize <= INT_MAX);
-        int elemOfs = charOfs / sizeof (double);
+        int elemOfs = (int)(charOfs / sizeof (double));
         partExts[modelID].start = elemOfs;
-        partExts[modelID].size = partSize;
+        partExts[modelID].size = (int)partSize;
         partExts[modelID].stride = 1;
         nmiss_ += dataHeader->nmiss;
       }
@@ -498,7 +496,7 @@ inventorizeStream(struct streamMapping *streamMap, int numStreamIDs,
   if (numStreamIDs < sizeStreamMap) ; else
     {
       streamMap = xrealloc(streamMap,
-                           (sizeStreamMap *= 2)
+                           (size_t)(sizeStreamMap *= 2)
                            * sizeof (streamMap[0]));
       *sizeStreamMap_ = sizeStreamMap;
     }
@@ -515,8 +513,7 @@ inventorizeStream(struct streamMapping *streamMap, int numStreamIDs,
       int nvars = vlistNvars(vlistID);
       streamMap[numStreamIDs].numVars = nvars;
       streamMap[numStreamIDs].varMap
-        = xmalloc(sizeof (streamMap[numStreamIDs].varMap[0])
-                  * nvars);
+        = xmalloc(sizeof (streamMap[numStreamIDs].varMap[0]) * (size_t)nvars);
       for (int i = 0; i < nvars; ++i)
         streamMap[numStreamIDs].varMap[i] = -1;
     }
@@ -538,10 +535,10 @@ buildStreamMap(struct winHeaderEntry *winDict)
 {
   int streamIDOld = CDI_UNDEFID;
   int oldStreamIdx = CDI_UNDEFID;
-  int filetype = CDI_UNDEFID;
+  int filetype = FILETYPE_UNDEF;
   int sizeStreamMap = 16;
   struct streamMapping *streamMap
-    = xmalloc(sizeStreamMap * sizeof (streamMap[0]));
+    = xmalloc((size_t)sizeStreamMap * sizeof (streamMap[0]));
   int numDataEntries = winDict[0].specific.headerSize.numDataEntries;
   int numStreamIDs = 0;
   /* find streams written on this process */
@@ -578,7 +575,7 @@ buildStreamMap(struct winHeaderEntry *winDict)
   {
     int *streamIDs, *streamIsWritten;
     int numTotalStreamIDs = streamSize();
-    streamIDs = xmalloc(2 * sizeof (streamIDs[0]) * (size_t)numTotalStreamIDs);
+    streamIDs = (int*) xmalloc(2 * sizeof (streamIDs[0]) * (size_t)numTotalStreamIDs);
     streamGetIndexList(numTotalStreamIDs, streamIDs);
     streamIsWritten = streamIDs + numTotalStreamIDs;
     for (int i = 0; i < numTotalStreamIDs; ++i)
@@ -597,8 +594,8 @@ buildStreamMap(struct winHeaderEntry *winDict)
     free(streamIDs);
   }
   /* sort written streams by streamID */
-  streamMap = xrealloc(streamMap, sizeof (streamMap[0]) * numStreamIDs);
-  qsort(streamMap, numStreamIDs, sizeof (streamMap[0]), smCmpStreamID);
+  streamMap = xrealloc(streamMap, sizeof (streamMap[0]) * (size_t)numStreamIDs);
+  qsort(streamMap, (size_t)numStreamIDs, sizeof (streamMap[0]), smCmpStreamID);
   return (struct streamMap){ .entries = streamMap, .numEntries = numStreamIDs };
 }
 
@@ -648,7 +645,7 @@ buildWrittenVars(struct streamMapping *mapping, int **varIsWritten_,
   int nvars = mapping->numVars;
   int *varMap = mapping->varMap;
   int *varIsWritten = *varIsWritten_
-    = xrealloc(*varIsWritten_, sizeof (*varIsWritten) * nvars);
+    = xrealloc(*varIsWritten_, sizeof (*varIsWritten) * (size_t)nvars);
   for (int varID = 0; varID < nvars; ++varID)
     varIsWritten[varID] = ((varMap[varID] != -1)
                            ?myCollRank+1 : 0);
@@ -795,9 +792,9 @@ static void readGetBuffers()
                       xt_idxlist_delete(preRedistChunk);
                       xt_idxlist_delete(preWriteChunk);
                       xt_xmap_delete(xmap);
-                      writeBuf = xrealloc(writeBuf,
-                                          sizeof (double)
-                                          * PPM_extents_size(3, varChunk));
+                      writeBuf = (double*) xrealloc(writeBuf,
+                                                    sizeof (double)
+                                                    * PPM_extents_size(3, varChunk));
                       xt_redist_s_exchange1(redist, data, writeBuf);
                       xt_redist_delete(redist);
                     }
@@ -911,11 +908,10 @@ void clearModelWinBuffer(int modelID)
 {
   int nProcsModel = commInqNProcsModel ();
 
-  xassert ( modelID                >= 0           &&
-            modelID                 < nProcsModel &&
-            rxWin != NULL && rxWin[modelID].buffer != NULL &&
-            rxWin[modelID].size > 0 &&
-            rxWin[modelID].size <= MAXWINBUFFERSIZE );
+  xassert((unsigned)modelID < (unsigned)nProcsModel &&
+          rxWin != NULL && rxWin[modelID].buffer != NULL &&
+          rxWin[modelID].size > 0 &&
+          rxWin[modelID].size <= MAXWINBUFFERSIZE );
   memset(rxWin[modelID].buffer, 0, rxWin[modelID].size);
 }
 
@@ -947,9 +943,9 @@ void getTimeStepData()
              modelID, nProcsModel, modelID, rxWin[modelID].size,
              getWinBaseAddr, (unsigned)sizeof(int));
       /* FIXME: this needs to use MPI_PACK for portability */
-      xmpi(MPI_Get(rxWin[modelID].buffer, rxWin[modelID].size,
+      xmpi(MPI_Get(rxWin[modelID].buffer, (int)rxWin[modelID].size,
                    MPI_UNSIGNED_CHAR, modelID, 0,
-                   rxWin[modelID].size, MPI_UNSIGNED_CHAR, getWin));
+                   (int)rxWin[modelID].size, MPI_UNSIGNED_CHAR, getWin));
     }
   xmpi ( MPI_Win_complete ( getWin ));
 
@@ -959,7 +955,7 @@ void getTimeStepData()
         sprintf(text, "rxWin[%d].size=%zu from PE%d rxWin[%d].buffer",
                 modelID, rxWin[modelID].size, modelID, modelID);
         xprintArray(text, rxWin[modelID].buffer,
-                    rxWin[modelID].size / sizeof (double),
+                    (int)(rxWin[modelID].size / sizeof (double)),
                     DATATYPE_FLT);
       }
   readGetBuffers();
@@ -980,13 +976,16 @@ cdiPioStreamCDFOpenWrap(const char *filename, const char *filemode,
     case FILETYPE_NC4:
     case FILETYPE_NC4C:
       {
-        int rank, fileID;
+        /* Only needs initialization to shut up gcc */
+        int rank = -1, fileID;
         int ioMode = commInqIOMode();
         if (ioMode == PIO_NONE
             || commInqRankColl() == (rank = cdiPioNextOpenRank()))
           fileID = cdiStreamOpenDefaultDelegate(filename, filemode, filetype,
                                                 streamptr,
                                                 recordBufIsToBeCreated);
+        else
+          streamptr->filetype = filetype;
         if (ioMode != PIO_NONE)
           xmpi(MPI_Bcast(&fileID, 1, MPI_INT, rank, commInqCommColl()));
         streamptr->ownerRank = rank;
@@ -1045,17 +1044,16 @@ cdiPioCdfDefTimestep(stream_t *streamptr, int tsID)
   @return
 */
 
-void IOServer(void (*postCommSetupActions)(void))
+void cdiPioServer(void (*postCommSetupActions)(void))
 {
-  int source, tag, size, nProcsModel=commInqNProcsModel();
+  int nProcsModel=commInqNProcsModel();
   static int nfinished = 0;
-  char * buffer;
   MPI_Comm commCalc;
   MPI_Status status;
 
   xdebug("%s", "START");
 
-  backendInit(postCommSetupActions);
+  cdiPioFileWritingInit(postCommSetupActions);
   if (commInqRankNode() == commInqSpecialRankNode())
     return;
   commCalc = commInqCommCalc ();
@@ -1065,7 +1063,7 @@ void IOServer(void (*postCommSetupActions)(void))
                                             &pioPrimes);
 #elif defined (HAVE_LIBNETCDF)
   cdiSerialOpenFileCount = xcalloc(sizeof (cdiSerialOpenFileCount[0]),
-                                   commInqSizeColl());
+                                   (size_t)commInqSizeColl());
   namespaceSwitchSet(NSSWITCH_STREAM_OPEN_BACKEND,
                      NSSW_FUNC(cdiPioStreamCDFOpenWrap));
   namespaceSwitchSet(NSSWITCH_STREAM_CLOSE_BACKEND,
@@ -1082,8 +1080,8 @@ void IOServer(void (*postCommSetupActions)(void))
     {
       xmpi ( MPI_Probe ( MPI_ANY_SOURCE, MPI_ANY_TAG, commCalc, &status ));
       
-      source = status.MPI_SOURCE;
-      tag    = status.MPI_TAG;
+      int source = status.MPI_SOURCE;
+      int tag = status.MPI_TAG;
       
       switch ( tag )
         {
@@ -1104,17 +1102,14 @@ void IOServer(void (*postCommSetupActions)(void))
 
                 if ( nStreams > 0 )
                   {
-                    int streamNo;
-                    int * resHs;
-
-                    resHs       = xmalloc ( nStreams * sizeof ( resHs[0] ));
+                    int * resHs = xmalloc((size_t)nStreams * sizeof (resHs[0]));
                     streamGetIndexList ( nStreams, resHs );
-                    for ( streamNo = 0; streamNo < nStreams; streamNo++ )
-                      streamClose ( resHs[streamNo] );
-                    free ( resHs );
+                    for (int streamNo = 0; streamNo < nStreams; ++streamNo)
+                      streamClose(resHs[streamNo]);
+                    free(resHs);
                   }
               }
-              backendCleanup();
+              cdiPioFileWritingFinalize();
               serverWinCleanup();
 #ifdef HAVE_PARALLEL_NC4
               free(pioPrimes);
@@ -1127,16 +1122,17 @@ void IOServer(void (*postCommSetupActions)(void))
           break;
           
 	case RESOURCES:
-	  xdebugMsg (  tag, source, nfinished );
-	  xmpi ( MPI_Get_count ( &status, MPI_CHAR, &size ));
-	  buffer = xmalloc(size);
-	  xmpi ( MPI_Recv ( buffer, size, MPI_PACKED, source,
-                            tag, commCalc, &status ));
-          xdebug("%s", "RECEIVED MESSAGE WITH TAG \"RESOURCES\"");
-	  reshUnpackResources(buffer, size, &commCalc);
-          xdebug("%s", "");
-	  free ( buffer );
           {
+            int size;
+            xdebugMsg(tag, source, nfinished);
+            xmpi(MPI_Get_count(&status, MPI_CHAR, &size));
+            char *buffer = xmalloc((size_t)size);
+            xmpi(MPI_Recv(buffer, size, MPI_PACKED, source,
+                          tag, commCalc, &status));
+            xdebug("%s", "RECEIVED MESSAGE WITH TAG \"RESOURCES\"");
+            reshUnpackResources(buffer, size, &commCalc);
+            xdebug("%s", "");
+            free(buffer);
             int rankGlob = commInqRankGlob();
             if ( ddebug > 0 && rankGlob == nProcsModel)
               {
@@ -1154,7 +1150,6 @@ void IOServer(void (*postCommSetupActions)(void))
           }
           serverWinCreate ();
 	  break;
-
 	case WRITETS:
           {
             xdebugMsg(tag, source, nfinished);
@@ -1172,7 +1167,6 @@ void IOServer(void (*postCommSetupActions)(void))
     }
 }
 
-#endif
 /*
  * Local Variables:
  * c-file-style: "Java"
