@@ -3,6 +3,7 @@
 #endif
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <float.h>
 #include <math.h>
@@ -16,6 +17,8 @@
 #include "varscan.h"
 #include "datetime.h"
 #include "ieg.h"
+#include "stream_fcommon.h"
+#include "stream_ieg.h"
 #include "vlist.h"
 
 
@@ -69,11 +72,10 @@ int iegInqRecord(stream_t *streamptr, int *varID, int *levelID)
   int icode, ilevel;
   int zaxisID = -1;
   int vlistID;
-  iegrec_t *iegp;
+  iegrec_t *iegp = (iegrec_t*) streamptr->record->exsep;
 
   vlistID = streamptr->vlistID;
   fileID  = streamptr->fileID;
-  iegp    = streamptr->record->iegp;
 
   *varID   = -1;
   *levelID = -1;
@@ -99,7 +101,7 @@ int iegInqRecord(stream_t *streamptr, int *varID, int *levelID)
 }
 */
 
-int iegReadRecord(stream_t *streamptr, double *data, int *nmiss)
+void iegReadRecord(stream_t *streamptr, double *data, int *nmiss)
 {
   int vlistID, fileID;
   int status;
@@ -108,7 +110,7 @@ int iegReadRecord(stream_t *streamptr, double *data, int *nmiss)
   int varID, gridID;
   int i, size;
   double missval;
-  iegrec_t *iegp;
+  iegrec_t *iegp = (iegrec_t*) streamptr->record->exsep;
 
   vlistID = streamptr->vlistID;
   fileID  = streamptr->fileID;
@@ -117,12 +119,12 @@ int iegReadRecord(stream_t *streamptr, double *data, int *nmiss)
   recID   = streamptr->tsteps[tsID].recIDs[vrecID];
   recpos  = streamptr->tsteps[tsID].records[recID].position;
   varID   = streamptr->tsteps[tsID].records[recID].varID;
-  iegp    = streamptr->record->iegp;
 
   fileSetPos(fileID, recpos, SEEK_SET);
 
   status = iegRead(fileID, iegp);
-  if ( status != 0 ) return (0);
+  if ( status != 0 )
+    Error("Could not read IEG record!");
 
   iegInqDataDP(iegp, data);
 
@@ -139,8 +141,6 @@ int iegReadRecord(stream_t *streamptr, double *data, int *nmiss)
 	data[i] = missval;
 	(*nmiss)++;
       }
-
-  return (1);
 }
 
 static
@@ -227,6 +227,32 @@ void iegDefTime(int *pdb, int date, int time, int taxisID)
   pdb[5] = 128;
 }
 
+static
+int calc_resfac(double xfirst, double xlast, double xinc, double yfirst, double ylast, double yinc)
+{
+  int i, j;
+  int iresfac = 1000;
+  int ifact;
+  int ifacarr[5] = {1000, 10000, 100000, 1000000, 10000000};
+  double vals[6] = {xfirst, xlast, xinc, yfirst, ylast, yinc};
+
+  for ( j = 0; j < 5; ++j )
+    {
+      ifact = ifacarr[j];
+      for ( i = 0; i < 6; ++i )
+        {
+          if ( fabs(vals[i]*ifact-(lround(vals[i]*ifact))) > FLT_EPSILON ) break;
+        }
+      if ( i == 6 )
+        {
+          iresfac = ifact;
+          break;
+        }
+    }
+
+  return (iresfac);
+}
+
 static
 void iegDefGrid(int *gdb, int gridID)
 {
@@ -242,7 +268,7 @@ void iegDefGrid(int *gdb, int gridID)
       ysize = gridInqYsize(gridID);
 
       if ( (ysize == 32  || ysize == 48 || ysize == 64 ||
-	    ysize == 96  || ysize == 160) && 
+	    ysize == 96  || ysize == 160) &&
 	   (xsize == 2*ysize || xsize == 1) )
 	{
 	  gridtype = GRID_GAUSSIAN;
@@ -302,31 +328,37 @@ void iegDefGrid(int *gdb, int gridID)
       else
 	IEG_G_GridType(gdb) = 0;
 
+      int iresfac = calc_resfac(xfirst, xlast, xinc, yfirst, ylast, yinc);
+      double resfac = (double) iresfac;
+      if ( iresfac == 1000 ) iresfac = 0;
+
+      IEG_G_ResFac(gdb)   = iresfac;
+
       IEG_G_NumLon(gdb)   = nlon;
       IEG_G_NumLat(gdb)   = nlat;
-      IEG_G_FirstLat(gdb) = NINT(yfirst*1000);
-      IEG_G_LastLat(gdb)  = NINT(ylast*1000);
-      IEG_G_FirstLon(gdb) = NINT(xfirst*1000);
-      IEG_G_LastLon(gdb)  = NINT(xlast*1000);
-      IEG_G_LonIncr(gdb)  = NINT(xinc*1000);
-      if ( fabs(xinc*1000 - IEG_G_LonIncr(gdb)) > FLT_EPSILON )
+      IEG_G_FirstLat(gdb) = lround(yfirst*resfac);
+      IEG_G_LastLat(gdb)  = lround(ylast*resfac);
+      IEG_G_FirstLon(gdb) = lround(xfirst*resfac);
+      IEG_G_LastLon(gdb)  = lround(xlast*resfac);
+      IEG_G_LonIncr(gdb)  = lround(xinc*resfac);
+      if ( fabs(xinc*resfac - IEG_G_LonIncr(gdb)) > FLT_EPSILON )
 	IEG_G_LonIncr(gdb) = 0;
 
       if ( gridtype == GRID_GAUSSIAN )
 	IEG_G_LatIncr(gdb) = nlat/2;
       else
 	{
-	  IEG_G_LatIncr(gdb) = NINT(yinc*1000);
-	  if ( fabs(yinc*1000 - IEG_G_LatIncr(gdb)) > FLT_EPSILON )
+	  IEG_G_LatIncr(gdb) = lround(yinc*resfac);
+	  if ( fabs(yinc*resfac - IEG_G_LatIncr(gdb)) > FLT_EPSILON )
 	    IEG_G_LatIncr(gdb) = 0;
 
 	  if ( IEG_G_LatIncr(gdb) < 0 ) IEG_G_LatIncr(gdb) = -IEG_G_LatIncr(gdb);
 	}
 
-      if ( IEG_G_NumLon(gdb) > 1 && IEG_G_NumLat(gdb) == 1 ) 
+      if ( IEG_G_NumLon(gdb) > 1 && IEG_G_NumLat(gdb) == 1 )
 	if ( IEG_G_LonIncr(gdb) != 0 && IEG_G_LatIncr(gdb) == 0 ) IEG_G_LatIncr(gdb) = IEG_G_LonIncr(gdb);
 
-      if ( IEG_G_NumLon(gdb) == 1 && IEG_G_NumLat(gdb) > 1 ) 
+      if ( IEG_G_NumLon(gdb) == 1 && IEG_G_NumLat(gdb) > 1 )
 	if ( IEG_G_LonIncr(gdb) == 0 && IEG_G_LatIncr(gdb) != 0 ) IEG_G_LonIncr(gdb) = IEG_G_LatIncr(gdb);
 
       if ( IEG_G_LatIncr(gdb) == 0 || IEG_G_LonIncr(gdb) == 0 )
@@ -336,8 +368,8 @@ void iegDefGrid(int *gdb, int gridID)
 
       if ( gridIsRotated(gridID) )
 	{
-	  IEG_G_LatSP(gdb) = - NINT(gridInqYpole(gridID) * 1000);
-	  IEG_G_LonSP(gdb) =   NINT((gridInqXpole(gridID) + 180) * 1000);
+	  IEG_G_LatSP(gdb) = - lround(gridInqYpole(gridID) * resfac);
+	  IEG_G_LonSP(gdb) =   lround((gridInqXpole(gridID) + 180) * resfac);
 	  IEG_G_Size(gdb)  = 42;
 	}
       else
@@ -420,8 +452,8 @@ void iegDefLevel(int *pdb, int *gdb, double *vct, int zaxisID, int levelID)
 	else
 	  {
 	    IEG_G_Size(gdb) += (vctsize*4);
-	    memcpy(vct, zaxisInqVctPtr(zaxisID), vctsize/2*sizeof(double));
-	    memcpy(vct+50, zaxisInqVctPtr(zaxisID)+vctsize/2, vctsize/2*sizeof(double));
+	    memcpy(vct, zaxisInqVctPtr(zaxisID), (size_t)vctsize/2*sizeof(double));
+	    memcpy(vct+50, zaxisInqVctPtr(zaxisID)+vctsize/2, (size_t)vctsize/2*sizeof(double));
 	  }
 	break;
       }
@@ -526,41 +558,14 @@ void iegDefLevel(int *pdb, int *gdb, double *vct, int zaxisID, int levelID)
 }
 
 
-int iegCopyRecord(stream_t *streamptr2, stream_t *streamptr1)
+void iegCopyRecord(stream_t *streamptr2, stream_t *streamptr1)
 {
-  int fileID1, fileID2;
-  int tsID, recID, vrecID;
-  long recsize;
-  off_t recpos;
-  int status = 0;
-  char *buffer;
-
-  fileID1 = streamptr1->fileID;
-  fileID2 = streamptr2->fileID;
-
-  tsID    = streamptr1->curTsID;
-  vrecID  = streamptr1->tsteps[tsID].curRecID;
-  recID   = streamptr1->tsteps[tsID].recIDs[vrecID];
-  recpos  = streamptr1->tsteps[tsID].records[recID].position;
-  recsize = streamptr1->tsteps[tsID].records[recID].size;
-
-  fileSetPos(fileID1, recpos, SEEK_SET);
-
-  buffer = (char *) malloc(recsize);
-
-  fileRead(fileID1, buffer, recsize);
-
-  fileWrite(fileID2, buffer, recsize);
-
-  free(buffer);
-
-  return (status);
+  streamFCopyRecord(streamptr2, streamptr1, "IEG");
 }
 
 
-int iegDefRecord(stream_t *streamptr)
+void iegDefRecord(stream_t *streamptr)
 {
-  int status = 0;
   int vlistID;
   int gridID;
   int date, time;
@@ -569,10 +574,9 @@ int iegDefRecord(stream_t *streamptr)
   int param, pdis, pcat, pnum;
   int varID, levelID, tsID, zaxisID;
   int byteorder;
-  iegrec_t *iegp;
+  iegrec_t *iegp = (iegrec_t*) streamptr->record->exsep;
 
   vlistID = streamptr->vlistID;
-  iegp    = streamptr->record->iegp;
   byteorder = streamptr->byteorder;
 
   varID   = streamptr->record->varID;
@@ -601,21 +605,17 @@ int iegDefRecord(stream_t *streamptr)
   datatype = streamptr->record->prec;
 
   iegp->dprec = iegDefDatatype(datatype);
-
-  return (status);
 }
 
 
-int iegWriteRecord(stream_t *streamptr, const double *data)
+void iegWriteRecord(stream_t *streamptr, const double *data)
 {
   int fileID;
-  int status = 0;
   int i, gridsize, gridID;
   double refval;
-  iegrec_t *iegp;
+  iegrec_t *iegp = (iegrec_t*) streamptr->record->exsep;
 
   fileID = streamptr->fileID;
-  iegp   = streamptr->record->iegp;
   gridID = streamptr->record->gridID;
 
   gridsize = gridInqSize(gridID);
@@ -629,13 +629,11 @@ int iegWriteRecord(stream_t *streamptr, const double *data)
   iegDefDataDP(iegp, data);
 
   iegWrite(fileID, iegp);
-
-  return (status);
 }
 
 static
 void iegAddRecord(stream_t *streamptr, int param, int *pdb, int *gdb, double *vct,
-		  long recsize, off_t position, int prec)
+		  size_t recsize, off_t position, int prec)
 {
   int leveltype;
   int gridID = UNDEFID;
@@ -666,12 +664,12 @@ void iegAddRecord(stream_t *streamptr, int param, int *pdb, int *gdb, double *vc
       if ( IEG_P_LevelType(pdb) == 100 ) level1 *= 100;
     }
 
-  (*record).size     = recsize;
-  (*record).position = position;
-  (*record).param    = param;
-  (*record).ilevel   = level1;
-  (*record).ilevel2  = level2;
-  (*record).ltype    = IEG_P_LevelType(pdb);
+  record->size     = recsize;
+  record->position = position;
+  record->param    = param;
+  record->ilevel   = level1;
+  record->ilevel2  = level2;
+  record->ltype    = IEG_P_LevelType(pdb);
 
   if ( IEG_G_GridType(gdb) == 0 || IEG_G_GridType(gdb) == 10 )
     gridtype = GRID_LONLAT;
@@ -681,27 +679,32 @@ void iegAddRecord(stream_t *streamptr, int param, int *pdb, int *gdb, double *vc
     gridtype = GRID_GENERIC;
 
   memset(&grid, 0, sizeof(grid_t));
-  grid.type  = gridtype; 
+  grid.type  = gridtype;
   grid.size  = IEG_G_NumLon(gdb)*IEG_G_NumLat(gdb);
   grid.xsize = IEG_G_NumLon(gdb);
   grid.ysize = IEG_G_NumLat(gdb);
   grid.xinc  = 0;
   grid.yinc  = 0;
   grid.xdef  = 0;
+
+  int iresfac = IEG_G_ResFac(gdb);
+  if ( iresfac == 0 ) iresfac = 1000;
+  double resfac = 1./(double) iresfac;
+
   /* if ( IEG_G_FirstLon != 0 || IEG_G_LastLon != 0 ) */
   {
     if ( grid.xsize > 1 )
       {
 	if ( IEG_G_ResFlag(gdb) && IEG_G_LonIncr(gdb) > 0 )
-	  grid.xinc = IEG_G_LonIncr(gdb) * 0.001;
+	  grid.xinc = IEG_G_LonIncr(gdb) * resfac;
 	else
-	  grid.xinc = (IEG_G_LastLon(gdb) - IEG_G_FirstLon(gdb)) * 0.001 / (grid.xsize - 1);
+	  grid.xinc = (IEG_G_LastLon(gdb) - IEG_G_FirstLon(gdb)) * resfac / (grid.xsize - 1);
 
 	/* correct xinc if necessary */
 	if ( IEG_G_FirstLon(gdb) == 0 && IEG_G_LastLon(gdb) > 354000 )
 	  {
 	    double xinc = 360. / grid.xsize;
-	    
+            /* FIXME: why not use grid.xinc != xinc as condition? */
 	    if ( fabs(grid.xinc-xinc) > 0.0 )
 	      {
 		grid.xinc = xinc;
@@ -709,9 +712,9 @@ void iegAddRecord(stream_t *streamptr, int param, int *pdb, int *gdb, double *vc
 	      }
 	  }
       }
-    grid.xfirst = IEG_G_FirstLon(gdb) * 0.001;
-    grid.xlast  = IEG_G_LastLon(gdb)  * 0.001;
-    grid.xdef   = 2;	    
+    grid.xfirst = IEG_G_FirstLon(gdb) * resfac;
+    grid.xlast  = IEG_G_LastLon(gdb)  * resfac;
+    grid.xdef   = 2;
   }
   grid.ydef  = 0;
   /* if ( IEG_G_FirstLat != 0 || IEG_G_LastLat != 0 ) */
@@ -719,22 +722,22 @@ void iegAddRecord(stream_t *streamptr, int param, int *pdb, int *gdb, double *vc
     if ( grid.ysize > 1 )
       {
 	if ( IEG_G_ResFlag(gdb) && IEG_G_LatIncr(gdb) > 0 )
-	  grid.yinc = IEG_G_LatIncr(gdb) * 0.001;
+	  grid.yinc = IEG_G_LatIncr(gdb) * resfac;
 	else
-	  grid.yinc = (IEG_G_LastLat(gdb) - IEG_G_FirstLat(gdb)) * 0.001 / (grid.ysize - 1);
+	  grid.yinc = (IEG_G_LastLat(gdb) - IEG_G_FirstLat(gdb)) * resfac / (grid.ysize - 1);
       }
-    grid.yfirst = IEG_G_FirstLat(gdb) * 0.001;
-    grid.ylast  = IEG_G_LastLat(gdb)  * 0.001;
-    grid.ydef   = 2;	    
+    grid.yfirst = IEG_G_FirstLat(gdb) * resfac;
+    grid.ylast  = IEG_G_LastLat(gdb)  * resfac;
+    grid.ydef   = 2;
   }
   /*
-  grid.xfirst= IEG_G_FirstLon(gdb) * 0.001;
-  grid.xlast = IEG_G_LastLon(gdb) * 0.001;
-  grid.xinc  = IEG_G_LonIncr(gdb) * 0.001;
-  grid.xdef  = 2;	    
-  grid.yfirst= IEG_G_FirstLat(gdb) * 0.001;
-  grid.ylast = IEG_G_LastLat(gdb) * 0.001;
-  grid.yinc  = IEG_G_LatIncr(gdb) * 0.001;
+  grid.xfirst= IEG_G_FirstLon(gdb) * resfac;
+  grid.xlast = IEG_G_LastLon(gdb) * resfac;
+  grid.xinc  = IEG_G_LonIncr(gdb) * resfac;
+  grid.xdef  = 2;
+  grid.yfirst= IEG_G_FirstLat(gdb) * resfac;
+  grid.ylast = IEG_G_LastLat(gdb) * resfac;
+  grid.yinc  = IEG_G_LatIncr(gdb) * resfac;
   grid.ydef  = 2;
   */
   grid.xvals = NULL;
@@ -744,8 +747,8 @@ void iegAddRecord(stream_t *streamptr, int param, int *pdb, int *gdb, double *vc
   if ( IEG_G_GridType(gdb) == 10 )
     {
       grid.isRotated = TRUE;
-      grid.ypole     = - IEG_G_LatSP(gdb) * 0.001;
-      grid.xpole     =   IEG_G_LonSP(gdb) * 0.001 - 180;
+      grid.ypole     = - IEG_G_LatSP(gdb) * resfac;
+      grid.xpole     =   IEG_G_LonSP(gdb) * resfac - 180;
       grid.angle     = 0;
     }
 
@@ -755,12 +758,11 @@ void iegAddRecord(stream_t *streamptr, int param, int *pdb, int *gdb, double *vc
 
   if ( leveltype == ZAXIS_HYBRID )
     {
-      int i;
       double tmpvct[100];
-      int vctsize = IEG_G_NumVCP(gdb);
+      size_t vctsize = (size_t)IEG_G_NumVCP(gdb);
 
-      for ( i = 0; i < vctsize/2; i++ ) tmpvct[i] = vct[i];
-      for ( i = 0; i < vctsize/2; i++ ) tmpvct[i+vctsize/2] = vct[i+50];
+      for (size_t i = 0; i < vctsize/2; i++ ) tmpvct[i] = vct[i];
+      for (size_t i = 0; i < vctsize/2; i++ ) tmpvct[i+vctsize/2] = vct[i+50];
 
       varDefVCT(vctsize, tmpvct);
     }
@@ -770,10 +772,10 @@ void iegAddRecord(stream_t *streamptr, int param, int *pdb, int *gdb, double *vc
   datatype = iegInqDatatype(prec);
 
   varAddRecord(recID, param, gridID, leveltype, lbounds, level1, level2, 0, 0,
-	       datatype, &varID, &levelID, UNDEFID, 0, 0, NULL, NULL, NULL);
+	       datatype, &varID, &levelID, TSTEP_INSTANT, 0, 0, -1, NULL, NULL, NULL, NULL);
 
-  (*record).varID   = varID;
-  (*record).levelID = levelID;
+  record->varID   = (short)varID;
+  record->levelID = (short)levelID;
 
   streamptr->tsteps[tsID].nallrecs++;
   streamptr->nrecs++;
@@ -842,18 +844,17 @@ void iegScanTimestep1(stream_t *streamptr)
   DateTime datetime, datetime0;
   int tsID;
   int varID;
-  long recsize;
+  size_t recsize;
   off_t recpos;
   int nrecords, nrecs, recID;
   int taxisID = -1;
   taxis_t *taxis;
   int vlistID;
   IEGCOMPVAR compVar, compVar0;
-  iegrec_t *iegp;
+  iegrec_t *iegp = (iegrec_t*) streamptr->record->exsep;
 
   streamptr->curTsID = 0;
 
-  iegp  = streamptr->record->iegp;
   tsID  = tstepsNewEntry(streamptr);
   taxis = &streamptr->tsteps[tsID].taxis;
 
@@ -872,7 +873,7 @@ void iegScanTimestep1(stream_t *streamptr)
 	  streamptr->ntsteps = 1;
 	  break;
 	}
-      recsize = fileGetPos(fileID) - recpos;
+      recsize = (size_t)(fileGetPos(fileID) - recpos);
 
       prec   = iegp->dprec;
       rcode  = IEG_P_Parameter(iegp->ipdb);
@@ -925,8 +926,8 @@ void iegScanTimestep1(stream_t *streamptr)
 
   taxisID = taxisCreate(TAXIS_ABSOLUTE);
   taxis->type  = TAXIS_ABSOLUTE;
-  taxis->vdate = datetime0.date;
-  taxis->vtime = datetime0.time;
+  taxis->vdate = (int)datetime0.date;
+  taxis->vtime = (int)datetime0.time;
 
   vlistID = streamptr->vlistID;
   vlistDefTaxis(vlistID, taxisID);
@@ -938,10 +939,11 @@ void iegScanTimestep1(stream_t *streamptr)
     {
       streamptr->tsteps[0].recordSize = nrecords;
       streamptr->tsteps[0].records =
-	(record_t *) realloc(streamptr->tsteps[0].records, nrecords*sizeof(record_t));
+	(record_t *)xrealloc(streamptr->tsteps[0].records,
+                             (size_t)nrecords * sizeof (record_t));
     }
 
-  streamptr->tsteps[0].recIDs = (int *) malloc(nrecords*sizeof(int));
+  streamptr->tsteps[0].recIDs = (int *)xmalloc((size_t)nrecords * sizeof (int));
   streamptr->tsteps[0].nrecs = nrecords;
   for ( recID = 0; recID < nrecords; recID++ )
     streamptr->tsteps[0].recIDs[recID] = recID;
@@ -979,20 +981,19 @@ int iegScanTimestep2(stream_t *streamptr)
   int rcode = 0, rlevel = 0, vdate = 0, vtime = 0;
   int tsID;
   int varID;
-  long recsize;
+  size_t recsize;
   off_t recpos = 0;
   int nrecords, nrecs, recID, rindex;
   int nextstep;
   taxis_t *taxis;
   int vlistID;
   IEGCOMPVAR compVar, compVar0;
-  iegrec_t *iegp;
+  iegrec_t *iegp = (iegrec_t*) streamptr->record->exsep;
 
   streamptr->curTsID = 1;
 
   vlistID = streamptr->vlistID;
   fileID  = streamptr->fileID;
-  iegp    = streamptr->record->iegp;
 
   tsID = streamptr->rtsteps;
   if ( tsID != 1 )
@@ -1005,7 +1006,7 @@ int iegScanTimestep2(stream_t *streamptr)
   cdi_create_records(streamptr, tsID);
 
   nrecords = streamptr->tsteps[0].nallrecs;
-  streamptr->tsteps[1].recIDs = (int *) malloc(nrecords*sizeof(int));
+  streamptr->tsteps[1].recIDs = (int *)xmalloc((size_t)nrecords * sizeof(int));
   streamptr->tsteps[1].nrecs = 0;
   for ( recID = 0; recID < nrecords; recID++ )
     streamptr->tsteps[1].recIDs[recID] = -1;
@@ -1028,7 +1029,7 @@ int iegScanTimestep2(stream_t *streamptr)
 	  streamptr->ntsteps = 2;
 	  break;
 	}
-      recsize = fileGetPos(fileID) - recpos;
+      recsize = (size_t)(fileGetPos(fileID) - recpos);
 
       rcode  = IEG_P_Parameter(iegp->ipdb);
       tabnum = IEG_P_CodeTable(iegp->ipdb);
@@ -1152,7 +1153,7 @@ int iegInqContents(stream_t *streamptr)
 }
 
 static
-int iegScanTimestep(stream_t *streamptr)
+long iegScanTimestep(stream_t *streamptr)
 {
   int status;
   int fileID;
@@ -1160,13 +1161,13 @@ int iegScanTimestep(stream_t *streamptr)
   int tabnum;
   int param = 0;
   int rcode = 0, rlevel = 0, vdate = 0, vtime = 0;
-  long recsize = 0;
+  size_t recsize = 0;
   off_t recpos = 0;
   int recID;
   taxis_t *taxis;
   int rindex, nrecs = 0;
   IEGCOMPVAR compVar, compVar0;
-  iegrec_t *iegp;
+  iegrec_t *iegp = (iegrec_t*) streamptr->record->exsep;
 
   if ( CDI_Debug )
     {
@@ -1179,7 +1180,6 @@ int iegScanTimestep(stream_t *streamptr)
   if ( streamptr->rtsteps == 0 )
     Error("Internal problem! Missing contents.");
 
-  iegp  = streamptr->record->iegp;
   tsID  = streamptr->rtsteps;
   taxis = &streamptr->tsteps[tsID].taxis;
 
@@ -1190,7 +1190,8 @@ int iegScanTimestep(stream_t *streamptr)
       nrecs = streamptr->tsteps[1].nrecs;
 
       streamptr->tsteps[tsID].nrecs = nrecs;
-      streamptr->tsteps[tsID].recIDs = (int *) malloc(nrecs*sizeof(int));
+      streamptr->tsteps[tsID].recIDs
+        = (int *)xmalloc((size_t)nrecs * sizeof (int));
       for ( recID = 0; recID < nrecs; recID++ )
 	streamptr->tsteps[tsID].recIDs[recID] = streamptr->tsteps[1].recIDs[recID];
 
@@ -1207,7 +1208,7 @@ int iegScanTimestep(stream_t *streamptr)
 	      streamptr->ntsteps = streamptr->rtsteps + 1;
 	      break;
 	    }
-	  recsize = fileGetPos(fileID) - recpos;
+	  recsize = (size_t)(fileGetPos(fileID) - recpos);
 
 	  rcode  = IEG_P_Parameter(iegp->ipdb);
 	  tabnum = IEG_P_CodeTable(iegp->ipdb);
@@ -1282,7 +1283,7 @@ int iegScanTimestep(stream_t *streamptr)
 
 int iegInqTimestep(stream_t *streamptr, int tsID)
 {
-  int ntsteps, nrecs;
+  int nrecs;
 
   if ( tsID == 0 && streamptr->rtsteps == 0 )
     Error("Call to cdiInqContents missing!");
@@ -1290,7 +1291,7 @@ int iegInqTimestep(stream_t *streamptr, int tsID)
   if ( CDI_Debug )
     Message("tsID = %d rtsteps = %d", tsID, streamptr->rtsteps);
 
-  ntsteps = UNDEFID;
+  long ntsteps = UNDEFID;
   while ( ( tsID + 1 ) > streamptr->rtsteps && ntsteps == UNDEFID )
     ntsteps = iegScanTimestep(streamptr);
 
@@ -1317,9 +1318,8 @@ void iegReadVarDP(stream_t *streamptr, int varID, double *data, int *nmiss)
   int recID;
   int i;
   double missval;
-  iegrec_t *iegp;
+  iegrec_t *iegp = (iegrec_t*) streamptr->record->exsep;
 
-  iegp     = streamptr->record->iegp;
   vlistID  = streamptr->vlistID;
   fileID   = streamptr->fileID;
   nlevs    = streamptr->vars[varID].nlevs;
@@ -1362,9 +1362,8 @@ void iegReadVarSliceDP(stream_t *streamptr, int varID, int levID, double *data,
   int recID;
   int i;
   double missval;
-  iegrec_t *iegp;
+  iegrec_t *iegp = (iegrec_t*) streamptr->record->exsep;
 
-  iegp     = streamptr->record->iegp;
   vlistID  = streamptr->vlistID;
   fileID   = streamptr->fileID;
   nlevs    = streamptr->vars[varID].nlevs;
@@ -1409,13 +1408,11 @@ void iegWriteVarDP(stream_t *streamptr, int varID, const double *data)
   int date, time;
   int param, pdis, pcat, pnum;
   double refval;
-  iegrec_t *iegp;
+  iegrec_t *iegp = (iegrec_t*) streamptr->record->exsep;
 
   if ( CDI_Debug )
     Message("streamID = %d  varID = %d", streamptr->self, varID);
 
-  iegp     = streamptr->record->iegp;
-
   iegInitMem(iegp);
   for ( i = 0; i < 37; i++ ) iegp->ipdb[i] = -1;
 
@@ -1470,14 +1467,14 @@ void iegWriteVarSliceDP(stream_t *streamptr, int varID, int levID, const double
   /* int tsID; */
   int vlistID;
   /* int param, date, time, datasize; */
-  iegrec_t *iegp;
+  iegrec_t *iegp = (iegrec_t*) streamptr->record->exsep;
 
-  iegp     = streamptr->record->iegp;
   vlistID  = streamptr->vlistID;
   fileID   = streamptr->fileID;
   /* tsID     = streamptr->curTsID; */
   gridID   = vlistInqVarGrid(vlistID, varID);
   zaxisID  = vlistInqVarZaxis(vlistID, varID);
+  (void)levID;
   /* level    = zaxisInqLevel(zaxisID, levID); */
 
   if ( CDI_Debug )
