@@ -2,7 +2,9 @@
 #  include "config.h"
 #endif
 
+#include <limits.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 
 #include "dmemory.h"
@@ -14,6 +16,8 @@
 #include "varscan.h"
 #include "datetime.h"
 #include "service.h"
+#include "stream_fcommon.h"
+#include "stream_srv.h"
 #include "vlist.h"
 
 
@@ -68,11 +72,10 @@ int srvInqRecord(stream_t *streamptr, int *varID, int *levelID)
   int zaxisID = -1;
   int header[8];
   int vlistID;
-  srvrec_t *srvp;
+  srvrec_t *srvp = (srvrec_t*) streamptr->record->exsep;
 
   vlistID = streamptr->vlistID;
   fileID  = streamptr->fileID;
-  srvp    = streamptr->record->srvp;
 
   *varID   = -1;
   *levelID = -1;
@@ -97,7 +100,7 @@ int srvInqRecord(stream_t *streamptr, int *varID, int *levelID)
 }
 */
 
-int srvReadRecord(stream_t *streamptr, double *data, int *nmiss)
+void srvReadRecord(stream_t *streamptr, double *data, int *nmiss)
 {
   int vlistID, fileID;
   int status;
@@ -107,7 +110,7 @@ int srvReadRecord(stream_t *streamptr, double *data, int *nmiss)
   int varID, gridID;
   int i, size;
   double missval;
-  srvrec_t *srvp;
+  srvrec_t *srvp = (srvrec_t*) streamptr->record->exsep;
 
   vlistID = streamptr->vlistID;
   fileID  = streamptr->fileID;
@@ -116,12 +119,12 @@ int srvReadRecord(stream_t *streamptr, double *data, int *nmiss)
   recID   = streamptr->tsteps[tsID].recIDs[vrecID];
   recpos  = streamptr->tsteps[tsID].records[recID].position;
   varID   = streamptr->tsteps[tsID].records[recID].varID;
-  srvp    = streamptr->record->srvp;
 
   fileSetPos(fileID, recpos, SEEK_SET);
 
   status = srvRead(fileID, srvp);
-  if ( status != 0 ) return (0);
+  if ( status != 0 )
+    Error("Failed to read record from SRV file");
 
   srvInqHeader(srvp, header);
   srvInqDataDP(srvp, data);
@@ -139,55 +142,25 @@ int srvReadRecord(stream_t *streamptr, double *data, int *nmiss)
 	data[i] = missval;
 	(*nmiss)++;
       }
-
-  return (1);
 }
 
 
-int srvCopyRecord(stream_t *streamptr2, stream_t *streamptr1)
+void srvCopyRecord(stream_t *streamptr2, stream_t *streamptr1)
 {
-  int fileID1, fileID2;
-  int tsID, recID, vrecID;
-  long recsize;
-  off_t recpos;
-  int status = 0;
-  char *buffer;
-
-  fileID1 = streamptr1->fileID;
-  fileID2 = streamptr2->fileID;
-
-  tsID    = streamptr1->curTsID;
-  vrecID  = streamptr1->tsteps[tsID].curRecID;
-  recID   = streamptr1->tsteps[tsID].recIDs[vrecID];
-  recpos  = streamptr1->tsteps[tsID].records[recID].position;
-  recsize = streamptr1->tsteps[tsID].records[recID].size;
-
-  fileSetPos(fileID1, recpos, SEEK_SET);
-
-  buffer = (char *) malloc(recsize);
-
-  fileRead(fileID1, buffer, recsize);
-
-  fileWrite(fileID2, buffer, recsize);
-
-  free(buffer);
-
-  return (status);
+  streamFCopyRecord(streamptr2, streamptr1, "SRV");
 }
 
 
-int srvDefRecord(stream_t *streamptr)
+void srvDefRecord(stream_t *streamptr)
 {
   int gridID;
   int header[8];
-  int status = 0;
   int xsize, ysize;
   int datatype;
   int pdis, pcat, pnum;
-  srvrec_t *srvp;
+  srvrec_t *srvp = (srvrec_t*) streamptr->record->exsep;
 
   gridID = streamptr->record->gridID;
-  srvp   = streamptr->record->srvp;
 
   cdiDecodeParam(streamptr->record->param, &pnum, &pcat, &pdis);
   header[0] = pnum;
@@ -216,33 +189,25 @@ int srvDefRecord(stream_t *streamptr)
   srvp->dprec = srvDefDatatype(datatype);
 
   srvDefHeader(srvp, header);
-
-  return (status);
 }
 
 
-int srvWriteRecord(stream_t *streamptr, const double *data)
+void srvWriteRecord(stream_t *streamptr, const double *data)
 {
-  int fileID;
-  int status = 0;
-  srvrec_t *srvp;
-
-  fileID = streamptr->fileID;
-  srvp   = streamptr->record->srvp;
+  int fileID = streamptr->fileID;
+  srvrec_t *srvp = (srvrec_t*) streamptr->record->exsep;
 
   srvDefDataDP(srvp, data);
 
   srvWrite(fileID, srvp);
-
-  return (status);
 }
 
 static
 void srv_add_record(stream_t *streamptr, int param, int level, int xsize, int ysize,
-                    long recsize, off_t position, int prec)
+                    size_t recsize, off_t position, int prec)
 {
   int leveltype;
-  int gridID = UNDEFID;
+  int gridID = CDI_UNDEFID;
   int levelID = 0;
   int tsID, recID, varID;
   int datatype;
@@ -277,10 +242,11 @@ void srv_add_record(stream_t *streamptr, int param, int level, int xsize, int ys
   datatype = srvInqDatatype(prec);
 
   varAddRecord(recID, param, gridID, leveltype, 0, level, 0, 0, 0,
-	       datatype, &varID, &levelID, UNDEFID, 0, 0, NULL, NULL, NULL);
+	       datatype, &varID, &levelID, TSTEP_INSTANT, 0, 0, -1, NULL, NULL, NULL, NULL);
 
-  (*record).varID   = varID;
-  (*record).levelID = levelID;
+  xassert(varID <= SHRT_MAX && levelID <= SHRT_MAX);
+  (*record).varID   = (short)varID;
+  (*record).levelID = (short)levelID;
 
   streamptr->tsteps[tsID].nallrecs++;
   streamptr->nrecs++;
@@ -290,33 +256,6 @@ void srv_add_record(stream_t *streamptr, int param, int level, int xsize, int ys
 	    varID, gridID, levelID);
 }
 
-
-void srvCmpRecord(stream_t *streamptr, int tsID, int recID, off_t position, int param,
-		  int level, int xsize, int ysize)
-{
-  int varID = 0;
-  int levelID = 0;
-  record_t *record;
-
-  record  = &streamptr->tsteps[tsID].records[recID];
-
-  if ( param != (*record).param || level != (*record).ilevel )
-    Error("inconsistent timestep");
-
-  (*record).position = position;
-  /*
-  varID   = (*record).varID;
-  levelID = (*record).levelID;
-
-  streamptr->vars[varID].level[levelID] = recID;
-
-  streamptr->tsteps[tsID].nallrecs++;
-  streamptr->nrecs++;
-  */
-  if ( CDI_Debug )
-    Message("varID = %d levelID = %d", varID, levelID);
-}
-
 static
 void srvScanTimestep1(stream_t *streamptr)
 {
@@ -330,18 +269,16 @@ void srvScanTimestep1(stream_t *streamptr)
   DateTime datetime, datetime0;
   int tsID;
   int varID;
-  long recsize;
   off_t recpos;
   int nrecords, nrecs, recID;
   int taxisID = -1;
   taxis_t *taxis;
   int vlistID;
   SRVCOMPVAR compVar, compVar0;
-  srvrec_t *srvp;
+  srvrec_t *srvp = (srvrec_t*) streamptr->record->exsep;
 
   streamptr->curTsID = 0;
 
-  srvp  = streamptr->record->srvp;
   tsID  = tstepsNewEntry(streamptr);
   taxis = &streamptr->tsteps[tsID].taxis;
 
@@ -360,7 +297,7 @@ void srvScanTimestep1(stream_t *streamptr)
 	  streamptr->ntsteps = 1;
 	  break;
 	}
-      recsize = fileGetPos(fileID) - recpos;
+      size_t recsize = (size_t)(fileGetPos(fileID) - recpos);
 
       srvInqHeader(srvp, header);
 
@@ -411,8 +348,8 @@ void srvScanTimestep1(stream_t *streamptr)
 
   taxisID = taxisCreate(TAXIS_ABSOLUTE);
   taxis->type  = TAXIS_ABSOLUTE;
-  taxis->vdate = datetime0.date;
-  taxis->vtime = datetime0.time;
+  taxis->vdate = (int)datetime0.date;
+  taxis->vtime = (int)datetime0.time;
 
   vlistID = streamptr->vlistID;
   vlistDefTaxis(vlistID, taxisID);
@@ -424,10 +361,11 @@ void srvScanTimestep1(stream_t *streamptr)
     {
       streamptr->tsteps[0].recordSize = nrecords;
       streamptr->tsteps[0].records =
-	(record_t *) realloc(streamptr->tsteps[0].records, nrecords*sizeof(record_t));
+	(record_t *)xrealloc(streamptr->tsteps[0].records,
+                             (size_t)nrecords * sizeof(record_t));
     }
 
-  streamptr->tsteps[0].recIDs = (int *) malloc(nrecords*sizeof(int));
+  streamptr->tsteps[0].recIDs = (int *)xmalloc((size_t)nrecords * sizeof (int));
   streamptr->tsteps[0].nrecs = nrecords;
   for ( recID = 0; recID < nrecords; recID++ )
     streamptr->tsteps[0].recIDs[recID] = recID;
@@ -465,20 +403,18 @@ int srvScanTimestep2(stream_t *streamptr)
   int rcode = 0, rlevel = 0, vdate = 0, vtime = 0;
   int tsID;
   int varID;
-  long recsize;
   off_t recpos = 0;
   int nrecords, nrecs, recID, rindex;
   int nextstep;
   taxis_t *taxis;
   int vlistID;
   SRVCOMPVAR compVar, compVar0;
-  srvrec_t *srvp;
+  srvrec_t *srvp = (srvrec_t*) streamptr->record->exsep;
 
   streamptr->curTsID = 1;
 
   vlistID = streamptr->vlistID;
   fileID  = streamptr->fileID;
-  srvp    = streamptr->record->srvp;
 
   tsID = streamptr->rtsteps;
   if ( tsID != 1 )
@@ -491,7 +427,7 @@ int srvScanTimestep2(stream_t *streamptr)
   cdi_create_records(streamptr, tsID);
 
   nrecords = streamptr->tsteps[0].nallrecs;
-  streamptr->tsteps[1].recIDs = (int *) malloc(nrecords*sizeof(int));
+  streamptr->tsteps[1].recIDs = (int *)xmalloc((size_t)nrecords * sizeof (int));
   streamptr->tsteps[1].nrecs = 0;
   for ( recID = 0; recID < nrecords; recID++ )
     streamptr->tsteps[1].recIDs[recID] = -1;
@@ -514,7 +450,7 @@ int srvScanTimestep2(stream_t *streamptr)
 	  streamptr->ntsteps = 2;
 	  break;
 	}
-      recsize = fileGetPos(fileID) - recpos;
+      size_t recsize = (size_t)(fileGetPos(fileID) - recpos);
 
       srvInqHeader(srvp, header);
 
@@ -632,23 +568,20 @@ int srvInqContents(stream_t *streamptr)
 }
 
 static
-int srvScanTimestep(stream_t *streamptr)
+long srvScanTimestep(stream_t *streamptr)
 {
   int header[8];
   int status;
   int fileID;
-  int tsID;
   /* int rxsize = 0, rysize = 0; */
   int param = 0;
   int rcode = 0, rlevel = 0, vdate = 0, vtime = 0;
-  long recsize = 0;
   off_t recpos = 0;
   int recID;
-  taxis_t *taxis;
   int rindex, nrecs = 0;
   SRVCOMPVAR compVar, compVar0;
-  srvrec_t *srvp;
-
+  srvrec_t *srvp = (srvrec_t*) streamptr->record->exsep;
+  /*
   if ( CDI_Debug )
     {
       Message("streamID = %d", streamptr->self);
@@ -656,13 +589,10 @@ int srvScanTimestep(stream_t *streamptr)
       Message("rts = %d", streamptr->rtsteps);
       Message("nts = %d", streamptr->ntsteps);
     }
+  */
 
-  if ( streamptr->rtsteps == 0 )
-    Error("Internal problem! Missing contents.");
-
-  srvp  = streamptr->record->srvp;
-  tsID  = streamptr->rtsteps;
-  taxis = &streamptr->tsteps[tsID].taxis;
+  int tsID  = streamptr->rtsteps;
+  taxis_t *taxis = &streamptr->tsteps[tsID].taxis;
 
   if ( streamptr->tsteps[tsID].recordSize == 0 )
     {
@@ -671,7 +601,7 @@ int srvScanTimestep(stream_t *streamptr)
       nrecs = streamptr->tsteps[1].nrecs;
 
       streamptr->tsteps[tsID].nrecs = nrecs;
-      streamptr->tsteps[tsID].recIDs = (int *) malloc(nrecs*sizeof(int));
+      streamptr->tsteps[tsID].recIDs = (int *)xmalloc((size_t)nrecs * sizeof (int));
       for ( recID = 0; recID < nrecs; recID++ )
 	streamptr->tsteps[tsID].recIDs[recID] = streamptr->tsteps[1].recIDs[recID];
 
@@ -688,7 +618,7 @@ int srvScanTimestep(stream_t *streamptr)
 	      streamptr->ntsteps = streamptr->rtsteps + 1;
 	      break;
 	    }
-	  recsize = fileGetPos(fileID) - recpos;
+	  size_t recsize = (size_t)(fileGetPos(fileID) - recpos);
 
 	  srvInqHeader(srvp, header);
 
@@ -711,9 +641,7 @@ int srvScanTimestep(stream_t *streamptr)
 	      taxis->vdate = vdate;
 	      taxis->vtime = vtime;
 	    }
-	  /*
-	  srvCmpRecord(streamptr, tsID, nrecs, recpos, param, rlevel, rxsize, rysize);
-	  */
+
 	  compVar.param  = param;
           compVar.level  = rlevel;
 	  compVar0.param = streamptr->tsteps[tsID].records[recID].param;
@@ -725,7 +653,7 @@ int srvScanTimestep(stream_t *streamptr)
 		      tsID, recID,
 		      streamptr->tsteps[tsID].records[recID].param, param,
 		      streamptr->tsteps[tsID].records[recID].ilevel, rlevel);
-	      Error("Invalid, unsupported or inconsistent record structure");
+	      Error("Invalid, unsupported or inconsistent record structure!");
 	    }
 
 	  streamptr->tsteps[tsID].records[recID].position = recpos;
@@ -763,7 +691,8 @@ int srvScanTimestep(stream_t *streamptr)
 
 int srvInqTimestep(stream_t *streamptr, int tsID)
 {
-  int ntsteps, nrecs;
+  long ntsteps;
+  int nrecs;
 
   if ( tsID == 0 && streamptr->rtsteps == 0 )
     Error("Call to cdiInqContents missing!");
@@ -775,7 +704,7 @@ int srvInqTimestep(stream_t *streamptr, int tsID)
   while ( ( tsID + 1 ) > streamptr->rtsteps && ntsteps == UNDEFID )
     ntsteps = srvScanTimestep(streamptr);
 
-  if ( tsID >= streamptr->ntsteps && streamptr->ntsteps != CDI_UNDEFID )
+  if ( tsID >= streamptr->ntsteps && streamptr->ntsteps != UNDEFID )
     {
       nrecs = 0;
     }
@@ -799,9 +728,8 @@ void srvReadVarDP(stream_t *streamptr, int varID, double *data, int *nmiss)
   int recID;
   int i;
   double missval;
-  srvrec_t *srvp;
+  srvrec_t *srvp = (srvrec_t*) streamptr->record->exsep;
 
-  srvp     = streamptr->record->srvp;
   vlistID  = streamptr->vlistID;
   fileID   = streamptr->fileID;
   nlevs    = streamptr->vars[varID].nlevs;
@@ -820,7 +748,8 @@ void srvReadVarDP(stream_t *streamptr, int varID, double *data, int *nmiss)
       recID = streamptr->vars[varID].level[levID];
       recpos = streamptr->tsteps[tsid].records[recID].position;
       fileSetPos(fileID, recpos, SEEK_SET);
-      srvRead(fileID, srvp);
+      if (srvRead(fileID, srvp) < 0)
+        abort();
       srvInqHeader(srvp, header);
       srvInqDataDP(srvp, &data[levID*gridsize]);
     }
@@ -846,9 +775,8 @@ void srvReadVarSliceDP(stream_t *streamptr, int varID, int levID, double *data,
   int recID;
   int i;
   double missval;
-  srvrec_t *srvp;
+  srvrec_t *srvp = (srvrec_t*) streamptr->record->exsep;
 
-  srvp     = streamptr->record->srvp;
   vlistID  = streamptr->vlistID;
   fileID   = streamptr->fileID;
   nlevs    = streamptr->vars[varID].nlevs;
@@ -866,7 +794,8 @@ void srvReadVarSliceDP(stream_t *streamptr, int varID, int levID, double *data,
   recID = streamptr->vars[varID].level[levID];
   recpos = streamptr->tsteps[tsid].records[recID].position;
   fileSetPos(fileID, recpos, SEEK_SET);
-  srvRead(fileID, srvp);
+  if (srvRead(fileID, srvp) < 0)
+    abort();
   srvInqHeader(srvp, header);
   srvInqDataDP(srvp, data);
 
@@ -894,12 +823,11 @@ void srvWriteVarDP(stream_t *streamptr, int varID, const double *data)
   int tsID;
   int vlistID;
   int pdis, pcat, pnum;
-  srvrec_t *srvp;
+  srvrec_t *srvp = (srvrec_t*) streamptr->record->exsep;
 
   if ( CDI_Debug )
     Message("streamID = %d  varID = %d", streamptr->self, varID);
 
-  srvp     = streamptr->record->srvp;
   vlistID  = streamptr->vlistID;
   fileID   = streamptr->fileID;
   tsID     = streamptr->curTsID;
@@ -961,9 +889,8 @@ void srvWriteVarSliceDP(stream_t *streamptr, int varID, int levID, const double
   int tsID;
   int vlistID;
   int pdis, pcat, pnum;
-  srvrec_t *srvp;
+  srvrec_t *srvp = (srvrec_t*) streamptr->record->exsep;
 
-  srvp     = streamptr->record->srvp;
   vlistID  = streamptr->vlistID;
   fileID   = streamptr->fileID;
   tsID     = streamptr->curTsID;
