@@ -14,7 +14,8 @@
 #include "zaxis.h"
 
 
-extern void zaxisGetIndexList ( int, int * );
+extern void zaxisGetIndexList(int, int *);
+extern void zaxisDefLtype2(int zaxisID, int ltype2);
 
 
 #undef  UNDEFID
@@ -25,7 +26,7 @@ static double *Vct = NULL;
 
 static int numberOfVerticalLevels = 0;
 static int numberOfVerticalGrid = 0;
-static char uuidVGrid[17];
+static unsigned char uuidVGrid[CDI_UUID_SIZE];
 
 typedef struct
 {
@@ -45,12 +46,13 @@ typedef struct
   int           timaccu;
   int           gridID;
   int           zaxistype;
-  int           ltype;     /* GRIB level type */
+  int           ltype1;     /* GRIB first level type */
+  int           ltype2;     /* GRIB second level type */
   int           lbounds;
   int           level_sf;
   int           level_unit;
   int           zaxisID;
-  int           nlevels;
+  unsigned      nlevels;
   int           levelTableSize;
   leveltable_t *levelTable;
   int           instID;
@@ -61,10 +63,12 @@ typedef struct
   int           lmissval;
   double        missval;
   char         *name;
+  char         *stdname;
   char         *longname;
   char         *units;
   ensinfo_t    *ensdata;
   int           typeOfGeneratingProcess;
+  int           productDefinitionTemplate;
 #if  defined  (HAVE_LIBGRIB_API)
   /* (Optional) list of keyword/double value pairs */
   int           opt_grib_dbl_nentries;
@@ -79,10 +83,9 @@ typedef struct
 vartable_t;
 
 
-int vartableInit = 0;
-vartable_t *vartable;
+static vartable_t *vartable;
 static int varTablesize = 0;
-int nvars = 0;
+static int nvars = 0;
 
 
 static
@@ -95,7 +98,8 @@ void paramInitEntry(int varID, int param)
   vartable[varID].timaccu        = 0;
   vartable[varID].gridID         = UNDEFID;
   vartable[varID].zaxistype      = 0;
-  vartable[varID].ltype          = 0;
+  vartable[varID].ltype1         = 0;
+  vartable[varID].ltype2         = -1;
   vartable[varID].lbounds        = 0;
   vartable[varID].level_sf       = 0;
   vartable[varID].level_unit     = 0;
@@ -105,27 +109,30 @@ void paramInitEntry(int varID, int param)
   vartable[varID].instID         = UNDEFID;
   vartable[varID].modelID        = UNDEFID;
   vartable[varID].tableID        = UNDEFID;
-  vartable[varID].typeOfGeneratingProcess  = UNDEFID;
+  vartable[varID].typeOfGeneratingProcess   = UNDEFID;
+  vartable[varID].productDefinitionTemplate = UNDEFID;
   vartable[varID].comptype       = COMPRESS_NONE;
   vartable[varID].complevel      = 1;
   vartable[varID].lmissval       = 0;
   vartable[varID].missval        = 0;
   vartable[varID].name           = NULL;
+  vartable[varID].stdname        = NULL;
   vartable[varID].longname       = NULL;
   vartable[varID].units          = NULL;
   vartable[varID].ensdata        = NULL;
 }
 
 static
-int varGetEntry(int param, int zaxistype, int ltype, const char *name)
+int varGetEntry(int param, int zaxistype, int ltype1, int tsteptype, const char *name)
 {
   int varID;
 
   for ( varID = 0; varID < varTablesize; varID++ )
     {
-      if ( vartable[varID].param     == param     &&
-	   vartable[varID].zaxistype == zaxistype &&
-	   vartable[varID].ltype     == ltype )
+      if ( vartable[varID].param      == param       &&
+	   vartable[varID].zaxistype  == zaxistype   &&
+	   vartable[varID].ltype1     == ltype1      &&
+	   vartable[varID].tsteptype  == tsteptype )
         {
           if ( name && name[0] && vartable[varID].name && vartable[varID].name[0] )
             {
@@ -152,6 +159,7 @@ void varFree(void)
 	free(vartable[varID].levelTable);
 
       if ( vartable[varID].name )     free(vartable[varID].name);
+      if ( vartable[varID].stdname )  free(vartable[varID].stdname);
       if ( vartable[varID].longname ) free(vartable[varID].longname);
       if ( vartable[varID].units )    free(vartable[varID].units);
       if ( vartable[varID].ensdata )  free(vartable[varID].ensdata);
@@ -190,7 +198,8 @@ int levelNewEntry(int varID, int level1, int level2)
       int i;
 
       levelTableSize = 2;
-      levelTable = (leveltable_t *) malloc(levelTableSize*sizeof(leveltable_t));
+      levelTable = (leveltable_t *)xmalloc((size_t)levelTableSize
+                                           * sizeof (leveltable_t));
       if( levelTable == NULL )
 	{
           Message("levelTableSize = %d", levelTableSize);
@@ -216,7 +225,8 @@ int levelNewEntry(int varID, int level1, int level2)
       int i;
 
       levelTableSize = 2*levelTableSize;
-      levelTable = (leveltable_t *) realloc(levelTable, levelTableSize*sizeof(leveltable_t));
+      levelTable = (leveltable_t *)xrealloc(levelTable, (size_t)levelTableSize
+                                            * sizeof (leveltable_t));
       if( levelTable == NULL )
 	{
           Message("levelTableSize = %d", levelTableSize);
@@ -232,7 +242,7 @@ int levelNewEntry(int varID, int level1, int level2)
   levelTable[levelID].level2   = level2;
   levelTable[levelID].lindex   = levelID;
 
-  vartable[varID].nlevels = levelID+1;
+  vartable[varID].nlevels = (unsigned)levelID+1;
   vartable[varID].levelTableSize = levelTableSize;
   vartable[varID].levelTable = levelTable;
 
@@ -255,7 +265,8 @@ int paramNewEntry(int param)
       int i;
 
       varTablesize = 2;
-      vartable = (vartable_t *) malloc(varTablesize*sizeof(vartable_t));
+      vartable = (vartable_t *)xmalloc((size_t)varTablesize
+                                       * sizeof (vartable_t));
       if( vartable == NULL )
 	{
           Message("varTablesize = %d", varTablesize);
@@ -286,8 +297,9 @@ int paramNewEntry(int param)
     {
       int i;
 
-      varTablesize = 2*varTablesize;
-      vartable = (vartable_t *) realloc(vartable, varTablesize*sizeof(vartable_t));
+      varTablesize = 2 * varTablesize;
+      vartable = (vartable_t *)xrealloc(vartable, (size_t)varTablesize
+                                        * sizeof (vartable_t));
       if( vartable == NULL )
 	{
           Message("varTablesize = %d", varTablesize);
@@ -313,29 +325,31 @@ int paramNewEntry(int param)
 
 void varAddRecord(int recID, int param, int gridID, int zaxistype, int lbounds,
 		  int level1, int level2, int level_sf, int level_unit, int prec,
-		  int *pvarID, int *plevelID, int tsteptype, int numavg, int ltype,
-		  const char *name, const char *longname, const char *units)
+		  int *pvarID, int *plevelID, int tsteptype, int numavg, int ltype1, int ltype2,
+		  const char *name, const char *stdname, const char *longname, const char *units)
 {
   int varID = UNDEFID;
   int levelID = -1;
 
   if ( ! (cdiSplitLtype105 == 1 && zaxistype == ZAXIS_HEIGHT) )
-    varID = varGetEntry(param, zaxistype, ltype, name);
+    varID = varGetEntry(param, zaxistype, ltype1, tsteptype, name);
 
   if ( varID == UNDEFID )
     {
       nvars++;
       varID = paramNewEntry(param);
-      vartable[varID].gridID    = gridID;
-      vartable[varID].zaxistype = zaxistype;
-      vartable[varID].ltype     = ltype;
-      vartable[varID].lbounds   = lbounds;
-      vartable[varID].level_sf  = level_sf;
+      vartable[varID].gridID     = gridID;
+      vartable[varID].zaxistype  = zaxistype;
+      vartable[varID].ltype1     = ltype1;
+      vartable[varID].ltype2     = ltype2;
+      vartable[varID].lbounds    = lbounds;
+      vartable[varID].level_sf   = level_sf;
       vartable[varID].level_unit = level_unit;
-      if ( tsteptype != UNDEFID ) vartable[varID].tsteptype = tsteptype;
+      vartable[varID].tsteptype  = tsteptype;
       if ( numavg ) vartable[varID].timave = 1;
 
       if ( name )     if ( name[0] )     vartable[varID].name     = strdup(name);
+      if ( stdname )  if ( stdname[0] )  vartable[varID].stdname  = strdup(stdname);
       if ( longname ) if ( longname[0] ) vartable[varID].longname = strdup(longname);
       if ( units )    if ( units[0] )    vartable[varID].units    = strdup(units);
     }
@@ -347,12 +361,12 @@ void varAddRecord(int recID, int param, int gridID, int zaxistype, int lbounds,
       if ( vartable[varID].gridID != gridID )
 	{
 	  Message("param = %s gridID = %d", paramstr, gridID);
-	  Error("horizontal grid must not change for same param!");
+	  Error("horizontal grid must not change for same parameter!");
 	}
       if ( vartable[varID].zaxistype != zaxistype )
 	{
 	  Message("param = %s zaxistype = %d", paramstr, zaxistype);
-	  Error("zaxistype must not change for same param!");
+	  Error("zaxistype must not change for same parameter!");
 	}
     }
 
@@ -361,9 +375,6 @@ void varAddRecord(int recID, int param, int gridID, int zaxistype, int lbounds,
   levelID = levelNewEntry(varID, level1, level2);
   vartable[varID].levelTable[levelID].recID = recID;
 
-  if ( CDI_Debug )
-    Message("varID = %d  levelID = %d", varID, levelID);
-
   *pvarID   = varID;
   *plevelID = levelID;
 }
@@ -383,8 +394,8 @@ static
 int cmpLevelTable(const void* s1, const void* s2)
 {
   int cmp = 0;
-  const leveltable_t* x = s1;
-  const leveltable_t* y = s2;
+  const leveltable_t* x = (const leveltable_t*) s1;
+  const leveltable_t* y = (const leveltable_t*) s2;
   /*
   printf("%g %g  %d %d\n", x->leve11, y->level1, x, y);
   */
@@ -398,8 +409,8 @@ static
 int cmpLevelTableInv(const void* s1, const void* s2)
 {
   int cmp = 0;
-  const leveltable_t* x = s1;
-  const leveltable_t* y = s2;
+  const leveltable_t* x = (const leveltable_t*) s1;
+  const leveltable_t* y = (const leveltable_t*) s2;
   /*
   printf("%g %g  %d %d\n", x->leve11, y->level1, x, y);
   */
@@ -422,26 +433,11 @@ param_t;
 static
 int cmpparam(const void* s1, const void* s2)
 {
-  int cmp = 0;
-  const param_t* x = s1;
-  const param_t* y = s2;
+  const param_t* x = (const param_t*) s1;
+  const param_t* y = (const param_t*) s2;
 
-  if      ( x->param > y->param ) cmp =  1;
-  else if ( x->param < y->param ) cmp = -1;
-
-  return (cmp);
-}
-
-
-static
-int cmpltype(const void* s1, const void* s2)
-{
-  int cmp = 0;
-  const param_t* x = s1;
-  const param_t* y = s2;
-
-  if      ( x->ltype > y->ltype ) cmp =  1;
-  else if ( x->ltype < y->ltype ) cmp = -1;
+  int cmp = (( x->param > y->param ) - ( x->param < y->param )) * 2
+           + ( x->ltype > y->ltype ) - ( x->ltype < y->ltype );
 
   return (cmp);
 }
@@ -449,9 +445,9 @@ int cmpltype(const void* s1, const void* s2)
 
 void cdi_generate_vars(stream_t *streamptr)
 {
-  int varID, gridID, zaxisID, levelID;
+  int varID, gridID, zaxisID;
   int instID, modelID, tableID;
-  int param, nlevels, zaxistype, lindex, ltype;
+  int param, zaxistype, ltype1, ltype2, lindex;
   int prec;
   int tsteptype;
   int timave, timaccu;
@@ -461,33 +457,29 @@ void cdi_generate_vars(stream_t *streamptr)
   double *dlevels = NULL;
   double *dlevels1 = NULL;
   double *dlevels2 = NULL;
-  int vlistID;
-  int *varids, index, varid;
+  int index, varid;
   double level_sf = 1;
+  int vlistID = streamptr->vlistID;
 
-  vlistID =  streamptr->vlistID;
-
-  varids = (int *) malloc(nvars*sizeof(int));
+  int *varids = (int *)xmalloc((size_t)nvars*sizeof(int));
   for ( varID = 0; varID < nvars; varID++ ) varids[varID] = varID;
 
   if ( streamptr->sortname )
     {
-      int index;
       param_t **varInfo;
-      varInfo    = (param_t **) malloc(nvars*sizeof(param_t *));
-      varInfo[0] = (param_t *)  malloc(nvars*sizeof(param_t));
+      varInfo    = (param_t **)xmalloc((size_t)nvars * sizeof (param_t *));
+      varInfo[0] = (param_t *)xmalloc((size_t)nvars * sizeof (param_t));
 
-      for ( index = 1; index < nvars; index++ )
+      for ( int index = 1; index < nvars; index++ )
 	varInfo[index] = varInfo[0] + index;
 
       for ( varid = 0; varid < nvars; varid++ )
 	{
 	  varInfo[varid]->varid = varids[varid];
 	  varInfo[varid]->param = vartable[varid].param;
-	  varInfo[varid]->ltype = vartable[varid].ltype;
+	  varInfo[varid]->ltype = vartable[varid].ltype1;
 	}
-      qsort(varInfo[0], nvars, sizeof(param_t), cmpltype);
-      qsort(varInfo[0], nvars, sizeof(param_t), cmpparam);
+      qsort(varInfo[0], (size_t)nvars, sizeof(param_t), cmpparam);
       for ( varid = 0; varid < nvars; varid++ )
 	{
 	  varids[varid] = varInfo[varid]->varid;
@@ -498,48 +490,49 @@ void cdi_generate_vars(stream_t *streamptr)
 
   for ( index = 0; index < nvars; index++ )
     {
-      varid     = varids[index];
+      varid      = varids[index];
 
-      gridID    = vartable[varid].gridID;
-      param     = vartable[varid].param;
-      nlevels   = vartable[varid].nlevels;
-      ltype     = vartable[varid].ltype;
+      gridID     = vartable[varid].gridID;
+      param      = vartable[varid].param;
+      unsigned nlevels = vartable[varid].nlevels;
+      ltype1     = vartable[varid].ltype1;
+      ltype2     = vartable[varid].ltype2;
       zaxistype = vartable[varid].zaxistype;
-      if ( ltype == 0 && zaxistype == ZAXIS_GENERIC && cdiDefaultLeveltype != -1 )
+      if ( ltype1 == 0 && zaxistype == ZAXIS_GENERIC && cdiDefaultLeveltype != -1 )
 	zaxistype = cdiDefaultLeveltype;
-      lbounds   = vartable[varid].lbounds;
-      prec      = vartable[varid].prec;
-      instID    = vartable[varid].instID;
-      modelID   = vartable[varid].modelID;
-      tableID   = vartable[varid].tableID;
-      tsteptype = vartable[varid].tsteptype;
-      timave    = vartable[varid].timave;
-      timaccu   = vartable[varid].timaccu;
-      comptype  = vartable[varid].comptype;
+      lbounds    = vartable[varid].lbounds;
+      prec       = vartable[varid].prec;
+      instID     = vartable[varid].instID;
+      modelID    = vartable[varid].modelID;
+      tableID    = vartable[varid].tableID;
+      tsteptype  = vartable[varid].tsteptype;
+      timave     = vartable[varid].timave;
+      timaccu    = vartable[varid].timaccu;
+      comptype   = vartable[varid].comptype;
 
       level_sf  = 1;
       if ( vartable[varid].level_sf != 0 ) level_sf = 1./vartable[varid].level_sf;
 
       zaxisID = UNDEFID;
 
-      if ( ltype == 0 && zaxistype == ZAXIS_GENERIC && nlevels == 1 &&
-	   ! (fabs(vartable[varid].levelTable[0].level1)>0) )
+      if ( ltype1 == 0 && zaxistype == ZAXIS_GENERIC && nlevels == 1 &&
+	   vartable[varid].levelTable[0].level1 == 0 )
 	zaxistype = ZAXIS_SURFACE;
 
       dlevels = (double *) malloc(nlevels*sizeof(double));
 
       if ( lbounds && zaxistype != ZAXIS_HYBRID && zaxistype != ZAXIS_HYBRID_HALF )
-	for ( levelID = 0; levelID < nlevels; levelID++ )
+	for (unsigned levelID = 0; levelID < nlevels; levelID++ )
 	  dlevels[levelID] = (level_sf*vartable[varid].levelTable[levelID].level1 +
 	                      level_sf*vartable[varid].levelTable[levelID].level2)/2;
       else
-	for ( levelID = 0; levelID < nlevels; levelID++ )
+	for (unsigned levelID = 0; levelID < nlevels; levelID++ )
 	  dlevels[levelID] = level_sf*vartable[varid].levelTable[levelID].level1;
 
       if ( nlevels > 1 )
 	{
           bool linc = true, ldec = true, lsort = false;
-          for ( levelID = 1; levelID < nlevels; levelID++ )
+          for (unsigned levelID = 1; levelID < nlevels; levelID++ )
             {
               /* check increasing of levels */
               linc &= (dlevels[levelID] > dlevels[levelID-1]);
@@ -553,7 +546,7 @@ void cdi_generate_vars(stream_t *streamptr)
            */
           if ( !ldec && zaxistype == ZAXIS_PRESSURE )
             {
-              qsort(vartable[varid].levelTable, (size_t)nlevels, sizeof(leveltable_t), cmpLevelTableInv);
+              qsort(vartable[varid].levelTable, nlevels, sizeof(leveltable_t), cmpLevelTableInv);
               lsort = true;
             }
           /*
@@ -565,18 +558,18 @@ void cdi_generate_vars(stream_t *streamptr)
                     zaxistype == ZAXIS_HYBRID ||
                     zaxistype == ZAXIS_DEPTH_BELOW_LAND )
             {
-              qsort(vartable[varid].levelTable, (size_t)nlevels, sizeof(leveltable_t), cmpLevelTable);
+              qsort(vartable[varid].levelTable, nlevels, sizeof(leveltable_t), cmpLevelTable);
               lsort = true;
             }
 
           if ( lsort )
             {
               if ( lbounds && zaxistype != ZAXIS_HYBRID && zaxistype != ZAXIS_HYBRID_HALF )
-                for ( levelID = 0; levelID < nlevels; levelID++ )
+                for (unsigned levelID = 0; levelID < nlevels; levelID++ )
                   dlevels[levelID] = (level_sf*vartable[varid].levelTable[levelID].level1 +
                                       level_sf*vartable[varid].levelTable[levelID].level2)/2.;
               else
-                for ( levelID = 0; levelID < nlevels; levelID++ )
+                for (unsigned levelID = 0; levelID < nlevels; levelID++ )
                   dlevels[levelID] = level_sf*vartable[varid].levelTable[levelID].level1;
             }
 	}
@@ -584,16 +577,21 @@ void cdi_generate_vars(stream_t *streamptr)
       if ( lbounds )
 	{
 	  dlevels1 = (double *) malloc(nlevels*sizeof(double));
-	  for ( levelID = 0; levelID < nlevels; levelID++ )
+	  for (unsigned levelID = 0; levelID < nlevels; levelID++)
 	    dlevels1[levelID] = level_sf*vartable[varid].levelTable[levelID].level1;
 	  dlevels2 = (double *) malloc(nlevels*sizeof(double));
-	  for ( levelID = 0; levelID < nlevels; levelID++ )
+	  for (unsigned levelID = 0; levelID < nlevels; levelID++)
 	    dlevels2[levelID] = level_sf*vartable[varid].levelTable[levelID].level2;
         }
 
       char *unitptr = cdiUnitNamePtr(vartable[varid].level_unit);
-      zaxisID = varDefZaxis(vlistID, zaxistype, nlevels, dlevels, lbounds, dlevels1, dlevels2,
-                            Vctsize, Vct, NULL, NULL, unitptr, 0, 0, ltype);
+      zaxisID = varDefZaxis(vlistID, zaxistype, (int)nlevels, dlevels, lbounds, dlevels1, dlevels2,
+                            (int)Vctsize, Vct, NULL, NULL, unitptr, 0, 0, ltype1);
+
+      if ( ltype1 != ltype2 && ltype2 != -1 )
+        {
+          zaxisDefLtype2(zaxisID, ltype2);
+        }
 
       if ( zaxisInqType(zaxisID) == ZAXIS_REFERENCE )
         {
@@ -618,9 +616,13 @@ void cdi_generate_vars(stream_t *streamptr)
       if ( vartable[varid].typeOfGeneratingProcess != UNDEFID )
         vlistDefVarTypeOfGeneratingProcess(vlistID, varID, vartable[varid].typeOfGeneratingProcess);
 
+      if ( vartable[varid].productDefinitionTemplate != UNDEFID )
+        vlistDefVarProductDefinitionTemplate(vlistID, varID, vartable[varid].productDefinitionTemplate);
+
       if ( vartable[varid].lmissval ) vlistDefVarMissval(vlistID, varID, vartable[varid].missval);
 
       if ( vartable[varid].name )     vlistDefVarName(vlistID, varID, vartable[varid].name);
+      if ( vartable[varid].stdname )  vlistDefVarStdname(vlistID, varID, vartable[varid].stdname);
       if ( vartable[varid].longname ) vlistDefVarLongname(vlistID, varID, vartable[varid].longname);
       if ( vartable[varid].units )    vlistDefVarUnits(vlistID, varID, vartable[varid].units);
 
@@ -641,6 +643,7 @@ void cdi_generate_vars(stream_t *streamptr)
           int idx = vlistptr->vars[varID].opt_grib_int_nentries;
           vlistptr->vars[varID].opt_grib_int_nentries++;
           if ( idx >= MAX_OPT_GRIB_ENTRIES ) Error("Too many optional keyword/integer value pairs!");
+          vlistptr->vars[varID].opt_grib_int_update[idx] = TRUE;
           vlistptr->vars[varID].opt_grib_int_val[idx] = vartable[varid].opt_grib_int_val[idx];
           vlistptr->vars[varID].opt_grib_int_keyword[idx] = strdupx(vartable[varid].opt_grib_int_keyword[idx]);
         }
@@ -649,6 +652,7 @@ void cdi_generate_vars(stream_t *streamptr)
           int idx = vlistptr->vars[varID].opt_grib_dbl_nentries;
           vlistptr->vars[varID].opt_grib_dbl_nentries++;
           if ( idx >= MAX_OPT_GRIB_ENTRIES ) Error("Too many optional keyword/double value pairs!");
+          vlistptr->vars[varID].opt_grib_dbl_update[idx] = TRUE;
           vlistptr->vars[varID].opt_grib_dbl_val[idx] = vartable[varid].opt_grib_dbl_val[idx];
           vlistptr->vars[varID].opt_grib_dbl_keyword[idx] = strdupx(vartable[varid].opt_grib_dbl_keyword[idx]);
         }
@@ -693,7 +697,7 @@ void cdi_generate_vars(stream_t *streamptr)
       varID     = index;
       varid     = varids[index];
 
-      nlevels   = vartable[varid].nlevels;
+      unsigned nlevels = vartable[varid].nlevels;
       /*
       for ( levelID = 0; levelID < nlevels; levelID++ )
 	{
@@ -704,16 +708,17 @@ void cdi_generate_vars(stream_t *streamptr)
 		 vartable[varid].levelTable[levelID].level1);
 	}
       */
-      for ( levelID = 0; levelID < nlevels; levelID++ )
+      for (unsigned levelID = 0; levelID < nlevels; levelID++)
 	{
 	  streamptr->vars[varID].level[levelID] = vartable[varid].levelTable[levelID].recID;
-	  for ( lindex = 0; lindex < nlevels; lindex++ )
-	    if ( levelID == vartable[varid].levelTable[lindex].lindex ) break;
+          unsigned lindex;
+	  for (lindex = 0; lindex < nlevels; lindex++ )
+	    if ( levelID == (unsigned)vartable[varid].levelTable[lindex].lindex ) break;
 
 	  if ( lindex == nlevels )
 	    Error("Internal problem! lindex not found.");
 
-	  streamptr->vars[varID].lindex[levelID] = lindex;
+	  streamptr->vars[varID].lindex[levelID] = (int)lindex;
 	}
     }
 
@@ -734,11 +739,11 @@ void varDefVCT(size_t vctsize, double *vctptr)
 }
 
 
-void varDefZAxisReference(int nhlev, int nvgrid, char *uuid)
+void varDefZAxisReference(int nhlev, int nvgrid, unsigned char uuid[CDI_UUID_SIZE])
 {
   numberOfVerticalLevels = nhlev;
   numberOfVerticalGrid = nvgrid;
-  memcpy(uuidVGrid, uuid, 16);
+  memcpy(uuidVGrid, uuid, CDI_UUID_SIZE);
 }
 
 
@@ -751,7 +756,7 @@ int varDefGrid(int vlistID, grid_t grid, int mode)
   int gridglobdefined = FALSE;
   int griddefined;
   int ngrids;
-  int gridID = UNDEFID;
+  int gridID = CDI_UNDEFID;
   int index;
   vlist_t *vlistptr;
   int * gridIndexList, i;
@@ -780,7 +785,7 @@ int varDefGrid(int vlistID, grid_t grid, int mode)
       ngrids = gridSize();
       if ( ngrids > 0 )
         {
-          gridIndexList = malloc(ngrids*sizeof(int));
+          gridIndexList = (int*) malloc(ngrids*sizeof(int));
           gridGetIndexList ( ngrids, gridIndexList );
           for ( i = 0; i < ngrids; i++ )
             {
@@ -816,14 +821,14 @@ int varDefGrid(int vlistID, grid_t grid, int mode)
 }
 
 
-int zaxisCompare(int zaxisID, int zaxistype, int nlevels, int lbounds, double *levels, char *longname, char *units, int ltype)
+int zaxisCompare(int zaxisID, int zaxistype, int nlevels, int lbounds, double *levels, char *longname, char *units, int ltype1)
 {
   int differ = 1;
   int levelID;
   int zlbounds = 0;
   int ltype_is_equal = FALSE;
 
-  if ( ltype == zaxisInqLtype(zaxisID) ) ltype_is_equal = TRUE;
+  if ( ltype1 == zaxisInqLtype(zaxisID) ) ltype_is_equal = TRUE;
 
   if ( ltype_is_equal && (zaxistype == zaxisInqType(zaxisID) || zaxistype == ZAXIS_GENERIC) )
     {
@@ -865,7 +870,7 @@ int zaxisCompare(int zaxisID, int zaxistype, int nlevels, int lbounds, double *l
 
 int varDefZaxis(int vlistID, int zaxistype, int nlevels, double *levels, int lbounds,
 		double *levels1, double *levels2, int vctsize, double *vct, char *name,
-		char *longname, char *units, int prec, int mode, int ltype)
+		char *longname, char *units, int prec, int mode, int ltype1)
 {
   /*
     mode: 0 search in vlist and zaxis table
@@ -889,7 +894,7 @@ int varDefZaxis(int vlistID, int zaxistype, int nlevels, double *levels, int lbo
       {
 	zaxisID = vlistptr->zaxisIDs[index];
 
-	if ( zaxisCompare(zaxisID, zaxistype, nlevels, lbounds, levels, longname, units, ltype) == 0 )
+	if ( zaxisCompare(zaxisID, zaxistype, nlevels, lbounds, levels, longname, units, ltype1) == 0 )
 	  {
 	    zaxisdefined = 1;
 	    break;
@@ -907,7 +912,7 @@ int varDefZaxis(int vlistID, int zaxistype, int nlevels, double *levels, int lbo
           for ( i = 0; i < nzaxis; i++ )
             {
               zaxisID = zaxisIndexList[i];
-              if ( zaxisCompare(zaxisID, zaxistype, nlevels, lbounds, levels, longname, units, ltype) == 0 )
+              if ( zaxisCompare(zaxisID, zaxistype, nlevels, lbounds, levels, longname, units, ltype1) == 0 )
                 {
                   zaxisglobdefined = 1;
                   break;
@@ -952,7 +957,7 @@ int varDefZaxis(int vlistID, int zaxistype, int nlevels, double *levels, int lbo
 	  zaxisDefLongname(zaxisID, longname);
 	  zaxisDefUnits(zaxisID, units);
 	  zaxisDefPrec(zaxisID, prec);
-	  zaxisDefLtype(zaxisID, ltype);
+	  zaxisDefLtype(zaxisID, ltype1);
 	}
 
       nzaxis = vlistptr->nzaxis;
@@ -1037,38 +1042,44 @@ void varDefTypeOfGeneratingProcess(int varID, int typeOfGeneratingProcess)
 }
 
 
+void varDefProductDefinitionTemplate(int varID, int productDefinitionTemplate)
+{
+  vartable[varID].productDefinitionTemplate = productDefinitionTemplate;
+}
+
+
+#if  defined  (HAVE_LIBGRIB_API)
 void varDefOptGribInt(int varID, long lval, const char *keyword)
 {
-#if  defined  (HAVE_LIBGRIB_API)
   int idx = vartable[varID].opt_grib_int_nentries;
   vartable[varID].opt_grib_int_nentries++;
   if ( idx >= MAX_OPT_GRIB_ENTRIES ) Error("Too many optional keyword/integer value pairs!");
   vartable[varID].opt_grib_int_val[idx] = (int) lval;
   vartable[varID].opt_grib_int_keyword[idx] = strdupx(keyword);
-#endif
 }
+#endif
 
 
+#if  defined  (HAVE_LIBGRIB_API)
 void varDefOptGribDbl(int varID, double dval, const char *keyword)
 {
-#if  defined  (HAVE_LIBGRIB_API)
   int idx = vartable[varID].opt_grib_dbl_nentries;
   vartable[varID].opt_grib_dbl_nentries++;
   if ( idx >= MAX_OPT_GRIB_ENTRIES ) Error("Too many optional keyword/double value pairs!");
   vartable[varID].opt_grib_dbl_val[idx] = dval;
   vartable[varID].opt_grib_dbl_keyword[idx] = strdupx(keyword);
-#endif
 }
+#endif
 
 
+#if  defined  (HAVE_LIBGRIB_API)
 int varOptGribNentries(int varID)
 {
   int nentries = 0;
-#if  defined  (HAVE_LIBGRIB_API)
   nentries = vartable[varID].opt_grib_int_nentries + vartable[varID].opt_grib_dbl_nentries;
-#endif
   return (nentries);
 }
+#endif
 
 /*
  * Local Variables:
