@@ -5,14 +5,15 @@
 #  include "grid.h"
 #endif
 
+#include "cdi.h"
 
 void varAddRecord(int recID, int param, int gridID, int zaxistype, int lbounds,
 		  int level1, int level2, int level_sf, int level_unit, int prec,
-		  int *pvarID, int *plevelID, int tsteptype, int numavg, int ltype,
-		  const char *name, const char *longname, const char *units);
+		  int *pvarID, int *plevelID, int tsteptype, int numavg, int ltype1, int ltype2,
+		  const char *name, const char *stdname, const char *longname, const char *units);
 
 void varDefVCT(size_t vctsize, double *vctptr);
-void varDefZAxisReference(int nlev, int nvgrid, char *uuid);
+void varDefZAxisReference(int nlev, int nvgrid, unsigned char uuid[CDI_UUID_SIZE]);
 
 int  varDefGrid(int vlistID, grid_t grid, int mode);
 int  varDefZaxis(int vlistID, int zaxistype, int nlevels, double *levels, int lbounds,
@@ -30,6 +31,7 @@ int  varInqTable(int varID);
 void varDefEnsembleInfo(int varID, int ens_idx, int ens_count, int forecast_type);
 
 void varDefTypeOfGeneratingProcess(int varID, int typeOfGeneratingProcess);
+void varDefProductDefinitionTemplate(int varID, int productDefinitionTemplate);
 
 
 void varDefOptGribInt(int varID, long lval, const char *keyword);
