@@ -139,7 +139,11 @@ void defineStream ( int streamID, int vlistID )
   streamDefVlist(streamID, vlistID);
 }
 
-int defineVlist ( int gridID, int zaxisID, int taxisID )
+struct idPair {
+  int id1, id2;
+};
+
+struct idPair defineVlist ( int gridID, int zaxisID, int taxisID )
 {
   int vlistID = CDI_UNDEFID;
   int zaxisID2 = zaxisCreate(ZAXIS_SURFACE, 1);
@@ -157,7 +161,10 @@ int defineVlist ( int gridID, int zaxisID, int taxisID )
   vlistDefVarName(vlistID, varID2, "varname2");
   vlistDefAttTxt(vlistID, varID2, "txt demo", 6, "banana");
   vlistDefTaxis(vlistID, taxisID);
-  return vlistID;
+  int vlistID2 = vlistCreate();
+  vlistDefVar(vlistID2, gridID, zaxisID, TIME_VARIABLE);
+  vlistCopy(vlistID2, vlistID);
+  return (struct idPair){ vlistID, vlistID2 };
 }
 
 int defineInstitute ()
@@ -173,7 +180,7 @@ int defineModel ( int instID )
 {
   int modelID = CDI_UNDEFID;
 
-  modelID = modelDef ( instID, 0, "myModel");
+  modelID = modelDef(instID, 0, "resource_copy");
 
   return modelID;
 }
@@ -182,33 +189,37 @@ static int destNamespace;
 
 int modelRun(MPI_Comm comm)
 {
-  int gridID, zaxisID, taxisID, instID, modelID, vlistID, streamID;
+  int gridID, zaxisID, taxisID, instID, vlistID, streamID;
 
   char * recvBuffer, * sendBuffer;
   int bufferSize, differ;
 
 #ifdef MPI_MARSHALLING
-  serializeSetMPI();
+  cdiPioSerializeSetMPI();
 #endif
 
   gridID  = defineGrid      ();
   zaxisID = defineZaxis     ();
   taxisID = defineTaxis     ();
   instID  = defineInstitute ();
-  modelID = defineModel     ( instID );
-  vlistID = defineVlist     ( gridID, zaxisID, taxisID);
-  streamID = streamOpenWrite("example.grb", FILETYPE_GRB);
-  if ( streamID < 0 ) xabort ( "Could not open file" );
-  defineStream ( streamID, vlistID );
+  defineModel(instID);
+  {
+    struct idPair temp = defineVlist(gridID, zaxisID, taxisID);
+    vlistID = temp.id1;
+    streamID = streamOpenWrite("example.grb", FILETYPE_GRB);
+    if ( streamID < 0 ) xabort ( "Could not open file" );
+    defineStream ( streamID, vlistID );
+    vlistDestroy(temp.id2);
+  }
 
   reshPackBufferCreate ( &sendBuffer, &bufferSize, &comm );
-  recvBuffer = xmalloc(bufferSize);
+  recvBuffer = xmalloc((size_t)bufferSize);
 #ifdef MPI_MARSHALLING
   xmpi(MPI_Sendrecv(sendBuffer, bufferSize, MPI_PACKED, 0, 0,
                     recvBuffer, bufferSize, MPI_PACKED, 0, 0,
                     MPI_COMM_SELF, MPI_STATUS_IGNORE));
 #else
-  memcpy(recvBuffer, sendBuffer, bufferSize);
+  memcpy(recvBuffer, sendBuffer, (size_t)bufferSize);
 #endif
   namespaceSetActive(destNamespace);
   reshUnpackResources(recvBuffer, bufferSize, &comm);
@@ -230,6 +241,7 @@ int main (int argc, char *argv[])
   MPI_Init(&argc, &argv);
   commModel = MPI_COMM_WORLD;
 #else
+  (void)argc; (void)argv;
   commModel = 0;
 #endif
   destNamespace = namespaceNew();
