@@ -83,15 +83,19 @@ void printAtts(FILE *fp, int vlistID, int varID)
 static
 void partab(FILE *fp, int vlistID, int option)
 {
-  int varID, code, tabnum, tableID, datatype = -1;
+  int varID, datatype = -1;
+  int param;
   char pstr[32];
+  char paramstr[32];
   char varname[CDI_MAX_NAME], varlongname[CDI_MAX_NAME], varstdname[CDI_MAX_NAME], varunits[CDI_MAX_NAME];
   int natts;
   int nvars;
   int chunktype;
+  int linebreak = 1;
   double missval;
       
   nvars  = vlistNvars(vlistID);
+  if ( option == 4 ) linebreak = 0;
 
   if ( option == 2 )
     {
@@ -119,24 +123,28 @@ void partab(FILE *fp, int vlistID, int option)
 
       if ( datatype != -1 )
 	{
-	  fprintf(fp, "&parameter\n");
-	  fprintf(fp, "  name=_default_\n");
+	  fprintf(fp, "&parameter");
+	  if ( linebreak ) fprintf(fp, "\n");
+	  fprintf(fp, "  name=_default_");
+	  if ( linebreak ) fprintf(fp, "\n");
 	  if ( datatype2str(datatype, pstr) == 0 )
-	    fprintf(fp, "  datatype=%s\n", pstr);
+	    {
+	      fprintf(fp, "  datatype=%s", pstr);
+	      if ( linebreak ) fprintf(fp, "\n");
+	    }
 	  fprintf(fp, "/\n");
 	}
     }
 
   for ( varID = 0; varID < nvars; varID++ )
     {
-      fprintf(fp, "&parameter\n");
+      fprintf(fp, "&parameter");
+      if ( linebreak ) fprintf(fp, "\n");
       
       varname[0]     = 0;
       varlongname[0] = 0;
       varunits[0]    = 0;
-      code     = vlistInqVarCode(vlistID, varID);
-      tableID  = vlistInqVarTable(vlistID, varID);
-      tabnum   = tableInqNum(tableID);
+      param    = vlistInqVarParam(vlistID, varID);
       missval  = vlistInqVarMissval(vlistID, varID);
       vlistInqVarName(vlistID, varID, varname);
       /* printf("1>%s<\n", varname); */
@@ -146,33 +154,61 @@ void partab(FILE *fp, int vlistID, int option)
       /* printf("3>%s<\n", varname); */
       vlistInqVarUnits(vlistID, varID, varunits);
             
-      fprintf(fp, "  name=%s\n", varname);
-      if ( code   > 0 ) fprintf(fp, "  code=%d\n", code);
-      if ( tabnum > 0 ) fprintf(fp, "  table=%d\n", tabnum);
+      fprintf(fp, "  name=%s", varname);
+      if ( linebreak ) fprintf(fp, "\n");
+      // if ( code   > 0 ) fprintf(fp, "  code=%d\n", code);
+      // if ( tabnum > 0 ) fprintf(fp, "  table=%d\n", tabnum);
+      if ( param >= 0 )
+	{
+	  cdiParamToString(param, paramstr, sizeof(paramstr));
+	  fprintf(fp, "  param=%s", paramstr);
+	  if ( linebreak ) fprintf(fp, "\n");
+	}
       if ( strlen(varstdname) )
-	fprintf(fp, "  standard_name=%s\n", varstdname);
+	{
+	  fprintf(fp, "  standard_name=%s", varstdname);
+	  if ( linebreak ) fprintf(fp, "\n");
+	}
       if ( strlen(varlongname) )
-	fprintf(fp, "  long_name=\"%s\"\n", varlongname);
+	{
+	  fprintf(fp, "  long_name=\"%s\"", varlongname);
+	  if ( linebreak ) fprintf(fp, "\n");
+	}
       if ( strlen(varunits) )
-	fprintf(fp, "  units=\"%s\"\n", varunits);
-      
+	{
+	  fprintf(fp, "  units=\"%s\"", varunits);
+	  if ( linebreak ) fprintf(fp, "\n");
+	}
+
       if ( datatype == -1 )
 	if ( datatype2str(vlistInqVarDatatype(vlistID, varID), pstr) == 0 )
-	  fprintf(fp, "  datatype=%s\n", pstr);
+	  {
+	    fprintf(fp, "  datatype=%s", pstr);
+	    if ( linebreak ) fprintf(fp, "\n");
+	  }
 
       chunktype = vlistInqVarChunkType(vlistID, varID);
       if ( chunktype == CHUNK_AUTO )
-	fprintf(fp, "  chunktype=auto\n");
+	{
+	  fprintf(fp, "  chunktype=auto");
+	  if ( linebreak ) fprintf(fp, "\n");
+	}
       else if ( chunktype == CHUNK_GRID )
-	fprintf(fp, "  chunktype=grid\n");
+	{
+	  fprintf(fp, "  chunktype=grid");
+	  if ( linebreak ) fprintf(fp, "\n");
+	}
       if ( chunktype == CHUNK_LINES )
-	fprintf(fp, "  chunktype=lines\n");
-	
+	{
+	  fprintf(fp, "  chunktype=lines");
+	  if ( linebreak ) fprintf(fp, "\n");
+	}
       
       if ( option == 2 ) printAtts(fp, vlistID, varID);
       if ( option == 2 ) 
 	fprintf(fp, "  missing_value=%g\n", missval);
       
+      if ( !linebreak ) fprintf(fp, "  ");
       fprintf(fp, "/\n");
     }   
 }
@@ -189,6 +225,9 @@ void filedes(int streamID)
     case FILETYPE_GRB:
       printf("  GRIB data\n");
       break;
+    case FILETYPE_GRB2:
+      printf("  GRIB2 data\n");
+      break;
     case FILETYPE_NC:
       printf("  netCDF data\n");
       break;
@@ -248,7 +287,7 @@ void filedes(int streamID)
 
 void *Filedes(void *argument)
 {
-  int GRIDDES, GRIDDES2, ZAXISDES, VCT, VCT2, PARDES, FILEDES, VLIST, PARTAB, PARTAB2;
+  int GRIDDES, GRIDDES2, ZAXISDES, VCT, VCT2, PARDES, FILEDES, VLIST, SPARTAB, PARTAB, PARTAB2;
   int operatorID;
   int streamID = 0;
   int zaxisID;
@@ -266,6 +305,7 @@ void *Filedes(void *argument)
   PARDES   = cdoOperatorAdd("pardes",    0, 0, NULL);
   FILEDES  = cdoOperatorAdd("filedes",   0, 0, NULL);
   VLIST    = cdoOperatorAdd("vlist",     0, 0, NULL);
+  SPARTAB  = cdoOperatorAdd("spartab",   0, 0, NULL);
   PARTAB   = cdoOperatorAdd("partab",    0, 0, NULL);
   PARTAB2  = cdoOperatorAdd("partab2",   0, 0, NULL);
 
@@ -309,6 +349,7 @@ void *Filedes(void *argument)
 		{
 		  if ( operatorID == VCT )
 		    {
+		      fprintf(stdout, "#   k         vct_a(k) [Pa]             vct_b(k) []\n");
 		      for ( i = 0; i < vctsize/2; i++ )
 			fprintf(stdout, "%5d %25.17f %25.17f\n", i, vct[i], vct[vctsize/2+i]);
 		    }
@@ -366,10 +407,11 @@ void *Filedes(void *argument)
 	  fprintf(stdout, "\n");
 	}   
     }
-  else if ( operatorID == PARTAB || operatorID == PARTAB2 )
+  else if ( operatorID == PARTAB || operatorID == SPARTAB || operatorID == PARTAB2 )
     {
       int option = 1;
 
+      if ( operatorID == SPARTAB ) option = 4;
       if ( operatorID == PARTAB2 ) option = 2;
       
       partab(stdout, vlistID, option);
