@@ -54,6 +54,8 @@ void *Splityear(void *argument)
 
   cdoInitialize(argument);
 
+  if ( processSelf() != 0 ) cdoAbort("This operator can't be combined with other operators!");
+
   if ( UNCHANGED_RECORD ) lcopy = TRUE;
 
   memset(cyear, 0, MAX_YEARS*sizeof(int));
@@ -74,7 +76,7 @@ void *Splityear(void *argument)
     {
       gridsize = vlistGridsizeMax(vlistID1);
       if ( vlistNumber(vlistID1) != CDI_REAL ) gridsize *= 2;
-      array = malloc(gridsize*sizeof(double));
+      array = (double*) malloc(gridsize*sizeof(double));
     }
 
   taxisID1 = vlistInqTaxis(vlistID1);
