@@ -314,7 +314,7 @@ void *Maggraph(void *argument);
 #define  ExprOperators          {"expr", "exprf", "aexpr", "aexprf"}
 #define  FCOperators            {"fc2sp", "sp2fc", "fc2gp", "gp2fc"}
 #define  FiledesOperators       {"filedes", "griddes", "griddes2", "zaxisdes", "vct", "vct2", "pardes", \
-                                 "vlist", "partab", "partab2"}
+                                 "vlist", "partab", "partab2", "spartab"}
 #define  FillmissOperators      {"fillmiss","fillmiss2"}
 #define  FilterOperators        {"bandpass", "highpass", "lowpass"}
 #define  FldrmsOperators        {"fldrms"}
